# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.enums",
    marshal="google.ads.googleads.v12",
    manifest={"RealEstatePlaceholderFieldEnum",},
)


class RealEstatePlaceholderFieldEnum(proto.Message):
    r"""Values for Real Estate placeholder fields.
    For more information about dynamic remarketing feeds, see
    https://support.google.com/google-ads/answer/6053288.

    """

    class RealEstatePlaceholderField(proto.Enum):
        r"""Possible values for Real Estate placeholder fields."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        LISTING_ID = 2
        LISTING_NAME = 3
        CITY_NAME = 4
        DESCRIPTION = 5
        ADDRESS = 6
        PRICE = 7
        FORMATTED_PRICE = 8
        IMAGE_URL = 9
        PROPERTY_TYPE = 10
        LISTING_TYPE = 11
        CONTEXTUAL_KEYWORDS = 12
        FINAL_URLS = 13
        FINAL_MOBILE_URLS = 14
        TRACKING_URL = 15
        ANDROID_APP_LINK = 16
        SIMILAR_LISTING_IDS = 17
        IOS_APP_LINK = 18
        IOS_APP_STORE_ID = 19


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.resources.types import (
    account_link as gagr_account_link,
)
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.services",
    marshal="google.ads.googleads.v12",
    manifest={
        "CreateAccountLinkRequest",
        "CreateAccountLinkResponse",
        "MutateAccountLinkRequest",
        "AccountLinkOperation",
        "MutateAccountLinkResponse",
        "MutateAccountLinkResult",
    },
)


class CreateAccountLinkRequest(proto.Message):
    r"""Request message for
    [AccountLinkService.CreateAccountLink][google.ads.googleads.v12.services.AccountLinkService.CreateAccountLink].

    Attributes:
        customer_id (str):
            Required. The ID of the customer for which
            the account link is created.
        account_link (google.ads.googleads.v12.resources.types.AccountLink):
            Required. The account link to be created.
    """

    customer_id = proto.Field(proto.STRING, number=1,)
    account_link = proto.Field(
        proto.MESSAGE, number=2, message=gagr_account_link.AccountLink,
    )


class CreateAccountLinkResponse(proto.Message):
    r"""Response message for
    [AccountLinkService.CreateAccountLink][google.ads.googleads.v12.services.AccountLinkService.CreateAccountLink].

    Attributes:
        resource_name (str):
            Returned for successful operations. Resource
            name of the account link.
    """

    resource_name = proto.Field(proto.STRING, number=1,)


class MutateAccountLinkRequest(proto.Message):
    r"""Request message for
    [AccountLinkService.MutateAccountLink][google.ads.googleads.v12.services.AccountLinkService.MutateAccountLink].

    Attributes:
        customer_id (str):
            Required. The ID of the customer being
            modified.
        operation (google.ads.googleads.v12.services.types.AccountLinkOperation):
            Required. The operation to perform on the
            link.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id = proto.Field(proto.STRING, number=1,)
    operation = proto.Field(
        proto.MESSAGE, number=2, message="AccountLinkOperation",
    )
    partial_failure = proto.Field(proto.BOOL, number=3,)
    validate_only = proto.Field(proto.BOOL, number=4,)


class AccountLinkOperation(proto.Message):
    r"""A single update on an account link.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        update (google.ads.googleads.v12.resources.types.AccountLink):
            Update operation: The account link is
            expected to have a valid resource name.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: A resource name for the account link to
            remove is expected, in this format:

            ``customers/{customer_id}/accountLinks/{account_link_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask = proto.Field(
        proto.MESSAGE, number=4, message=field_mask_pb2.FieldMask,
    )
    update = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=gagr_account_link.AccountLink,
    )
    remove = proto.Field(proto.STRING, number=3, oneof="operation",)


class MutateAccountLinkResponse(proto.Message):
    r"""Response message for account link mutate.

    Attributes:
        result (google.ads.googleads.v12.services.types.MutateAccountLinkResult):
            Result for the mutate.
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
    """

    result = proto.Field(
        proto.MESSAGE, number=1, message="MutateAccountLinkResult",
    )
    partial_failure_error = proto.Field(
        proto.MESSAGE, number=2, message=status_pb2.Status,
    )


class MutateAccountLinkResult(proto.Message):
    r"""The result for the account link mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name = proto.Field(proto.STRING, number=1,)


__all__ = tuple(sorted(__protobuf__.manifest))



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Example Airflow DAG that show how to use various Looker
operators to submit PDT materialization job and manage it.
"""
from __future__ import annotations

from datetime import datetime

from airflow import models
from airflow.providers.google.cloud.operators.looker import LookerStartPdtBuildOperator
from airflow.providers.google.cloud.sensors.looker import LookerCheckPdtBuildSensor

with models.DAG(
    dag_id="example_gcp_looker",
    start_date=datetime(2021, 1, 1),
    catchup=False,
) as dag:
    # [START cloud_looker_async_start_pdt_sensor]
    start_pdt_task_async = LookerStartPdtBuildOperator(
        task_id="start_pdt_task_async",
        looker_conn_id="your_airflow_connection_for_looker",
        model="your_lookml_model",
        view="your_lookml_view",
        asynchronous=True,
    )

    check_pdt_task_async_sensor = LookerCheckPdtBuildSensor(
        task_id="check_pdt_task_async_sensor",
        looker_conn_id="your_airflow_connection_for_looker",
        materialization_id=start_pdt_task_async.output,
        poke_interval=10,
    )
    # [END cloud_looker_async_start_pdt_sensor]

    # [START how_to_cloud_looker_start_pdt_build_operator]
    build_pdt_task = LookerStartPdtBuildOperator(
        task_id="build_pdt_task",
        looker_conn_id="your_airflow_connection_for_looker",
        model="your_lookml_model",
        view="your_lookml_view",
    )
    # [END how_to_cloud_looker_start_pdt_build_operator]

    start_pdt_task_async >> check_pdt_task_async_sensor

    build_pdt_task



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.errors",
    marshal="google.ads.googleads.v12",
    manifest={"CampaignErrorEnum",},
)


class CampaignErrorEnum(proto.Message):
    r"""Container for enum describing possible campaign errors.
    """

    class CampaignError(proto.Enum):
        r"""Enum describing possible campaign errors."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        CANNOT_TARGET_CONTENT_NETWORK = 3
        CANNOT_TARGET_SEARCH_NETWORK = 4
        CANNOT_TARGET_SEARCH_NETWORK_WITHOUT_GOOGLE_SEARCH = 5
        CANNOT_TARGET_GOOGLE_SEARCH_FOR_CPM_CAMPAIGN = 6
        CAMPAIGN_MUST_TARGET_AT_LEAST_ONE_NETWORK = 7
        CANNOT_TARGET_PARTNER_SEARCH_NETWORK = 8
        CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CRITERIA_LEVEL_BIDDING_STRATEGY = (
            9
        )
        CAMPAIGN_DURATION_MUST_CONTAIN_ALL_RUNNABLE_TRIALS = 10
        CANNOT_MODIFY_FOR_TRIAL_CAMPAIGN = 11
        DUPLICATE_CAMPAIGN_NAME = 12
        INCOMPATIBLE_CAMPAIGN_FIELD = 13
        INVALID_CAMPAIGN_NAME = 14
        INVALID_AD_SERVING_OPTIMIZATION_STATUS = 15
        INVALID_TRACKING_URL = 16
        CANNOT_SET_BOTH_TRACKING_URL_TEMPLATE_AND_TRACKING_SETTING = 17
        MAX_IMPRESSIONS_NOT_IN_RANGE = 18
        TIME_UNIT_NOT_SUPPORTED = 19
        INVALID_OPERATION_IF_SERVING_STATUS_HAS_ENDED = 20
        BUDGET_CANNOT_BE_SHARED = 21
        CAMPAIGN_CANNOT_USE_SHARED_BUDGET = 22
        CANNOT_CHANGE_BUDGET_ON_CAMPAIGN_WITH_TRIALS = 23
        CAMPAIGN_LABEL_DOES_NOT_EXIST = 24
        CAMPAIGN_LABEL_ALREADY_EXISTS = 25
        MISSING_SHOPPING_SETTING = 26
        INVALID_SHOPPING_SALES_COUNTRY = 27
        ADVERTISING_CHANNEL_TYPE_NOT_AVAILABLE_FOR_ACCOUNT_TYPE = 31
        INVALID_ADVERTISING_CHANNEL_SUB_TYPE = 32
        AT_LEAST_ONE_CONVERSION_MUST_BE_SELECTED = 33
        CANNOT_SET_AD_ROTATION_MODE = 34
        CANNOT_MODIFY_START_DATE_IF_ALREADY_STARTED = 35
        CANNOT_SET_DATE_TO_PAST = 36
        MISSING_HOTEL_CUSTOMER_LINK = 37
        INVALID_HOTEL_CUSTOMER_LINK = 38
        MISSING_HOTEL_SETTING = 39
        CANNOT_USE_SHARED_CAMPAIGN_BUDGET_WHILE_PART_OF_CAMPAIGN_GROUP = 40
        APP_NOT_FOUND = 41
        SHOPPING_ENABLE_LOCAL_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE = 42
        MERCHANT_NOT_ALLOWED_FOR_COMPARISON_LISTING_ADS = 43
        INSUFFICIENT_APP_INSTALLS_COUNT = 44
        SENSITIVE_CATEGORY_APP = 45
        HEC_AGREEMENT_REQUIRED = 46
        NOT_COMPATIBLE_WITH_VIEW_THROUGH_CONVERSION_OPTIMIZATION = 49
        INVALID_EXCLUDED_PARENT_ASSET_FIELD_TYPE = 48
        CANNOT_CREATE_APP_PRE_REGISTRATION_FOR_NON_ANDROID_APP = 50
        APP_NOT_AVAILABLE_TO_CREATE_APP_PRE_REGISTRATION_CAMPAIGN = 51
        INCOMPATIBLE_BUDGET_TYPE = 52
        LOCAL_SERVICES_DUPLICATE_CATEGORY_BID = 53
        LOCAL_SERVICES_INVALID_CATEGORY_BID = 54
        LOCAL_SERVICES_MISSING_CATEGORY_BID = 55


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.common.types import criteria
from airflow.providers.google_vendor.googleads.v12.common.types import extensions
from airflow.providers.google_vendor.googleads.v12.enums.types import keyword_match_type
from airflow.providers.google_vendor.googleads.v12.enums.types import recommendation_type
from airflow.providers.google_vendor.googleads.v12.enums.types import (
    target_cpa_opt_in_recommendation_goal,
)
from airflow.providers.google_vendor.googleads.v12.resources.types import ad as gagr_ad


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.resources",
    marshal="google.ads.googleads.v12",
    manifest={"Recommendation",},
)


class Recommendation(proto.Message):
    r"""A recommendation.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the recommendation.

            ``customers/{customer_id}/recommendations/{recommendation_id}``
        type_ (google.ads.googleads.v12.enums.types.RecommendationTypeEnum.RecommendationType):
            Output only. The type of recommendation.
        impact (google.ads.googleads.v12.resources.types.Recommendation.RecommendationImpact):
            Output only. The impact on account
            performance as a result of applying the
            recommendation.
        campaign_budget (str):
            Output only. The budget targeted by this recommendation.
            This will be set only when the recommendation affects a
            single campaign budget.

            This field will be set for the following recommendation
            types: CAMPAIGN_BUDGET, FORECASTING_CAMPAIGN_BUDGET,
            MARGINAL_ROI_CAMPAIGN_BUDGET, MOVE_UNUSED_BUDGET

            This field is a member of `oneof`_ ``_campaign_budget``.
        campaign (str):
            Output only. The campaign targeted by this recommendation.
            This will be set only when the recommendation affects a
            single campaign.

            This field will be set for the following recommendation
            types: CALL_EXTENSION, CALLOUT_EXTENSION,
            ENHANCED_CPC_OPT_IN, USE_BROAD_MATCH_KEYWORD, KEYWORD,
            KEYWORD_MATCH_TYPE,
            UPGRADE_LOCAL_CAMPAIGN_TO_PERFORMANCE_MAX,
            MAXIMIZE_CLICKS_OPT_IN, MAXIMIZE_CONVERSIONS_OPT_IN,
            OPTIMIZE_AD_ROTATION, RESPONSIVE_SEARCH_AD,
            RESPONSIVE_SEARCH_AD_ASSET, SEARCH_PARTNERS_OPT_IN,
            DISPLAY_EXPANSION_OPT_IN, SITELINK_EXTENSION,
            TARGET_CPA_OPT_IN, TARGET_ROAS_OPT_IN, TEXT_AD,
            UPGRADE_SMART_SHOPPING_CAMPAIGN_TO_PERFORMANCE_MAX ,
            RAISE_TARGET_CPA_BID_TOO_LOW, FORECASTING_SET_TARGET_ROAS

            This field is a member of `oneof`_ ``_campaign``.
        ad_group (str):
            Output only. The ad group targeted by this recommendation.
            This will be set only when the recommendation affects a
            single ad group.

            This field will be set for the following recommendation
            types: KEYWORD, OPTIMIZE_AD_ROTATION, RESPONSIVE_SEARCH_AD,
            RESPONSIVE_SEARCH_AD_ASSET, TEXT_AD

            This field is a member of `oneof`_ ``_ad_group``.
        dismissed (bool):
            Output only. Whether the recommendation is
            dismissed or not.

            This field is a member of `oneof`_ ``_dismissed``.
        campaign_budget_recommendation (google.ads.googleads.v12.resources.types.Recommendation.CampaignBudgetRecommendation):
            Output only. The campaign budget
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        forecasting_campaign_budget_recommendation (google.ads.googleads.v12.resources.types.Recommendation.CampaignBudgetRecommendation):
            Output only. The forecasting campaign budget
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        keyword_recommendation (google.ads.googleads.v12.resources.types.Recommendation.KeywordRecommendation):
            Output only. The keyword recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        text_ad_recommendation (google.ads.googleads.v12.resources.types.Recommendation.TextAdRecommendation):
            Output only. Add expanded text ad
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        target_cpa_opt_in_recommendation (google.ads.googleads.v12.resources.types.Recommendation.TargetCpaOptInRecommendation):
            Output only. The TargetCPA opt-in
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        maximize_conversions_opt_in_recommendation (google.ads.googleads.v12.resources.types.Recommendation.MaximizeConversionsOptInRecommendation):
            Output only. The MaximizeConversions Opt-In
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        enhanced_cpc_opt_in_recommendation (google.ads.googleads.v12.resources.types.Recommendation.EnhancedCpcOptInRecommendation):
            Output only. The Enhanced Cost-Per-Click
            Opt-In recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        search_partners_opt_in_recommendation (google.ads.googleads.v12.resources.types.Recommendation.SearchPartnersOptInRecommendation):
            Output only. The Search Partners Opt-In
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        maximize_clicks_opt_in_recommendation (google.ads.googleads.v12.resources.types.Recommendation.MaximizeClicksOptInRecommendation):
            Output only. The MaximizeClicks Opt-In
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        optimize_ad_rotation_recommendation (google.ads.googleads.v12.resources.types.Recommendation.OptimizeAdRotationRecommendation):
            Output only. The Optimize Ad Rotation
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        callout_extension_recommendation (google.ads.googleads.v12.resources.types.Recommendation.CalloutExtensionRecommendation):
            Output only. The Callout extension
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        sitelink_extension_recommendation (google.ads.googleads.v12.resources.types.Recommendation.SitelinkExtensionRecommendation):
            Output only. The Sitelink extension
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        call_extension_recommendation (google.ads.googleads.v12.resources.types.Recommendation.CallExtensionRecommendation):
            Output only. The Call extension
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        keyword_match_type_recommendation (google.ads.googleads.v12.resources.types.Recommendation.KeywordMatchTypeRecommendation):
            Output only. The keyword match type
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        move_unused_budget_recommendation (google.ads.googleads.v12.resources.types.Recommendation.MoveUnusedBudgetRecommendation):
            Output only. The move unused budget
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        target_roas_opt_in_recommendation (google.ads.googleads.v12.resources.types.Recommendation.TargetRoasOptInRecommendation):
            Output only. The Target ROAS opt-in
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        responsive_search_ad_recommendation (google.ads.googleads.v12.resources.types.Recommendation.ResponsiveSearchAdRecommendation):
            Output only. The add responsive search ad
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        marginal_roi_campaign_budget_recommendation (google.ads.googleads.v12.resources.types.Recommendation.CampaignBudgetRecommendation):
            Output only. The marginal ROI campaign budget
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        use_broad_match_keyword_recommendation (google.ads.googleads.v12.resources.types.Recommendation.UseBroadMatchKeywordRecommendation):
            Output only. The use broad match keyword
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        responsive_search_ad_asset_recommendation (google.ads.googleads.v12.resources.types.Recommendation.ResponsiveSearchAdAssetRecommendation):
            Output only. The responsive search ad asset
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        upgrade_smart_shopping_campaign_to_performance_max_recommendation (google.ads.googleads.v12.resources.types.Recommendation.UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation):
            Output only. The upgrade a Smart Shopping
            campaign to a Performance Max campaign
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        responsive_search_ad_improve_ad_strength_recommendation (google.ads.googleads.v12.resources.types.Recommendation.ResponsiveSearchAdImproveAdStrengthRecommendation):
            Output only. The responsive search ad improve
            ad strength recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        display_expansion_opt_in_recommendation (google.ads.googleads.v12.resources.types.Recommendation.DisplayExpansionOptInRecommendation):
            Output only. The Display Expansion opt-in
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        upgrade_local_campaign_to_performance_max_recommendation (google.ads.googleads.v12.resources.types.Recommendation.UpgradeLocalCampaignToPerformanceMaxRecommendation):
            Output only. The upgrade a Local campaign to
            a Performance Max campaign recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        raise_target_cpa_bid_too_low_recommendation (google.ads.googleads.v12.resources.types.Recommendation.RaiseTargetCpaBidTooLowRecommendation):
            Output only. The raise target CPA bid too low
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
        forecasting_set_target_roas_recommendation (google.ads.googleads.v12.resources.types.Recommendation.ForecastingSetTargetRoasRecommendation):
            Output only. The forecasting set target ROAS
            recommendation.

            This field is a member of `oneof`_ ``recommendation``.
    """

    class RecommendationImpact(proto.Message):
        r"""The impact of making the change as described in the
        recommendation. Some types of recommendations may not have
        impact information.

        Attributes:
            base_metrics (google.ads.googleads.v12.resources.types.Recommendation.RecommendationMetrics):
                Output only. Base metrics at the time the
                recommendation was generated.
            potential_metrics (google.ads.googleads.v12.resources.types.Recommendation.RecommendationMetrics):
                Output only. Estimated metrics if the
                recommendation is applied.
        """

        base_metrics = proto.Field(
            proto.MESSAGE,
            number=1,
            message="Recommendation.RecommendationMetrics",
        )
        potential_metrics = proto.Field(
            proto.MESSAGE,
            number=2,
            message="Recommendation.RecommendationMetrics",
        )

    class RecommendationMetrics(proto.Message):
        r"""Weekly account performance metrics. For some recommendation
        types, these are averaged over the past 90-day period and hence
        can be fractional.

        Attributes:
            impressions (float):
                Output only. Number of ad impressions.

                This field is a member of `oneof`_ ``_impressions``.
            clicks (float):
                Output only. Number of ad clicks.

                This field is a member of `oneof`_ ``_clicks``.
            cost_micros (int):
                Output only. Cost (in micros) for
                advertising, in the local currency for the
                account.

                This field is a member of `oneof`_ ``_cost_micros``.
            conversions (float):
                Output only. Number of conversions.

                This field is a member of `oneof`_ ``_conversions``.
            video_views (float):
                Output only. Number of video views for a
                video ad campaign.

                This field is a member of `oneof`_ ``_video_views``.
        """

        impressions = proto.Field(proto.DOUBLE, number=6, optional=True,)
        clicks = proto.Field(proto.DOUBLE, number=7, optional=True,)
        cost_micros = proto.Field(proto.INT64, number=8, optional=True,)
        conversions = proto.Field(proto.DOUBLE, number=9, optional=True,)
        video_views = proto.Field(proto.DOUBLE, number=10, optional=True,)

    class CampaignBudgetRecommendation(proto.Message):
        r"""The budget recommendation for budget constrained campaigns.

        Attributes:
            current_budget_amount_micros (int):
                Output only. The current budget amount in
                micros.

                This field is a member of `oneof`_ ``_current_budget_amount_micros``.
            recommended_budget_amount_micros (int):
                Output only. The recommended budget amount in
                micros.

                This field is a member of `oneof`_ ``_recommended_budget_amount_micros``.
            budget_options (Sequence[google.ads.googleads.v12.resources.types.Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption]):
                Output only. The budget amounts and
                associated impact estimates for some values of
                possible budget amounts.
        """

        class CampaignBudgetRecommendationOption(proto.Message):
            r"""The impact estimates for a given budget amount.

            Attributes:
                budget_amount_micros (int):
                    Output only. The budget amount for this
                    option.

                    This field is a member of `oneof`_ ``_budget_amount_micros``.
                impact (google.ads.googleads.v12.resources.types.Recommendation.RecommendationImpact):
                    Output only. The impact estimate if budget is
                    changed to amount specified in this option.
            """

            budget_amount_micros = proto.Field(
                proto.INT64, number=3, optional=True,
            )
            impact = proto.Field(
                proto.MESSAGE,
                number=2,
                message="Recommendation.RecommendationImpact",
            )

        current_budget_amount_micros = proto.Field(
            proto.INT64, number=7, optional=True,
        )
        recommended_budget_amount_micros = proto.Field(
            proto.INT64, number=8, optional=True,
        )
        budget_options = proto.RepeatedField(
            proto.MESSAGE,
            number=3,
            message="Recommendation.CampaignBudgetRecommendation.CampaignBudgetRecommendationOption",
        )

    class KeywordRecommendation(proto.Message):
        r"""The keyword recommendation.

        Attributes:
            keyword (google.ads.googleads.v12.common.types.KeywordInfo):
                Output only. The recommended keyword.
            recommended_cpc_bid_micros (int):
                Output only. The recommended CPC
                (cost-per-click) bid.

                This field is a member of `oneof`_ ``_recommended_cpc_bid_micros``.
        """

        keyword = proto.Field(
            proto.MESSAGE, number=1, message=criteria.KeywordInfo,
        )
        recommended_cpc_bid_micros = proto.Field(
            proto.INT64, number=3, optional=True,
        )

    class TextAdRecommendation(proto.Message):
        r"""The text ad recommendation.

        Attributes:
            ad (google.ads.googleads.v12.resources.types.Ad):
                Output only. Recommended ad.
            creation_date (str):
                Output only. Creation date of the recommended
                ad. YYYY-MM-DD format, for example, 2018-04-17.

                This field is a member of `oneof`_ ``_creation_date``.
            auto_apply_date (str):
                Output only. Date, if present, is the
                earliest when the recommendation will be auto
                applied. YYYY-MM-DD format, for example,
                2018-04-17.

                This field is a member of `oneof`_ ``_auto_apply_date``.
        """

        ad = proto.Field(proto.MESSAGE, number=1, message=gagr_ad.Ad,)
        creation_date = proto.Field(proto.STRING, number=4, optional=True,)
        auto_apply_date = proto.Field(proto.STRING, number=5, optional=True,)

    class TargetCpaOptInRecommendation(proto.Message):
        r"""The Target CPA opt-in recommendation.

        Attributes:
            options (Sequence[google.ads.googleads.v12.resources.types.Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption]):
                Output only. The available goals and
                corresponding options for Target CPA strategy.
            recommended_target_cpa_micros (int):
                Output only. The recommended average CPA
                target. See required budget amount and impact of
                using this recommendation in options list.

                This field is a member of `oneof`_ ``_recommended_target_cpa_micros``.
        """

        class TargetCpaOptInRecommendationOption(proto.Message):
            r"""The Target CPA opt-in option with impact estimate.

            Attributes:
                goal (google.ads.googleads.v12.enums.types.TargetCpaOptInRecommendationGoalEnum.TargetCpaOptInRecommendationGoal):
                    Output only. The goal achieved by this
                    option.
                target_cpa_micros (int):
                    Output only. Average CPA target.

                    This field is a member of `oneof`_ ``_target_cpa_micros``.
                required_campaign_budget_amount_micros (int):
                    Output only. The minimum campaign budget, in
                    local currency for the account, required to
                    achieve the target CPA. Amount is specified in
                    micros, where one million is equivalent to one
                    currency unit.

                    This field is a member of `oneof`_ ``_required_campaign_budget_amount_micros``.
                impact (google.ads.googleads.v12.resources.types.Recommendation.RecommendationImpact):
                    Output only. The impact estimate if this
                    option is selected.
            """

            goal = proto.Field(
                proto.ENUM,
                number=1,
                enum=target_cpa_opt_in_recommendation_goal.TargetCpaOptInRecommendationGoalEnum.TargetCpaOptInRecommendationGoal,
            )
            target_cpa_micros = proto.Field(
                proto.INT64, number=5, optional=True,
            )
            required_campaign_budget_amount_micros = proto.Field(
                proto.INT64, number=6, optional=True,
            )
            impact = proto.Field(
                proto.MESSAGE,
                number=4,
                message="Recommendation.RecommendationImpact",
            )

        options = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="Recommendation.TargetCpaOptInRecommendation.TargetCpaOptInRecommendationOption",
        )
        recommended_target_cpa_micros = proto.Field(
            proto.INT64, number=3, optional=True,
        )

    class MaximizeConversionsOptInRecommendation(proto.Message):
        r"""The Maximize Conversions Opt-In recommendation.

        Attributes:
            recommended_budget_amount_micros (int):
                Output only. The recommended new budget
                amount.

                This field is a member of `oneof`_ ``_recommended_budget_amount_micros``.
        """

        recommended_budget_amount_micros = proto.Field(
            proto.INT64, number=2, optional=True,
        )

    class EnhancedCpcOptInRecommendation(proto.Message):
        r"""The Enhanced Cost-Per-Click Opt-In recommendation.
        """

    class SearchPartnersOptInRecommendation(proto.Message):
        r"""The Search Partners Opt-In recommendation.
        """

    class MaximizeClicksOptInRecommendation(proto.Message):
        r"""The Maximize Clicks opt-in recommendation.

        Attributes:
            recommended_budget_amount_micros (int):
                Output only. The recommended new budget
                amount. Only set if the current budget is too
                high.

                This field is a member of `oneof`_ ``_recommended_budget_amount_micros``.
        """

        recommended_budget_amount_micros = proto.Field(
            proto.INT64, number=2, optional=True,
        )

    class OptimizeAdRotationRecommendation(proto.Message):
        r"""The Optimize Ad Rotation recommendation.
        """

    class CalloutExtensionRecommendation(proto.Message):
        r"""The Callout extension recommendation.

        Attributes:
            recommended_extensions (Sequence[google.ads.googleads.v12.common.types.CalloutFeedItem]):
                Output only. Callout extensions recommended
                to be added.
        """

        recommended_extensions = proto.RepeatedField(
            proto.MESSAGE, number=1, message=extensions.CalloutFeedItem,
        )

    class SitelinkExtensionRecommendation(proto.Message):
        r"""The Sitelink extension recommendation.

        Attributes:
            recommended_extensions (Sequence[google.ads.googleads.v12.common.types.SitelinkFeedItem]):
                Output only. Sitelink extensions recommended
                to be added.
        """

        recommended_extensions = proto.RepeatedField(
            proto.MESSAGE, number=1, message=extensions.SitelinkFeedItem,
        )

    class CallExtensionRecommendation(proto.Message):
        r"""The Call extension recommendation.

        Attributes:
            recommended_extensions (Sequence[google.ads.googleads.v12.common.types.CallFeedItem]):
                Output only. Call extensions recommended to
                be added.
        """

        recommended_extensions = proto.RepeatedField(
            proto.MESSAGE, number=1, message=extensions.CallFeedItem,
        )

    class KeywordMatchTypeRecommendation(proto.Message):
        r"""The keyword match type recommendation.

        Attributes:
            keyword (google.ads.googleads.v12.common.types.KeywordInfo):
                Output only. The existing keyword where the
                match type should be more broad.
            recommended_match_type (google.ads.googleads.v12.enums.types.KeywordMatchTypeEnum.KeywordMatchType):
                Output only. The recommended new match type.
        """

        keyword = proto.Field(
            proto.MESSAGE, number=1, message=criteria.KeywordInfo,
        )
        recommended_match_type = proto.Field(
            proto.ENUM,
            number=2,
            enum=keyword_match_type.KeywordMatchTypeEnum.KeywordMatchType,
        )

    class MoveUnusedBudgetRecommendation(proto.Message):
        r"""The move unused budget recommendation.

        Attributes:
            excess_campaign_budget (str):
                Output only. The excess budget's resource_name.

                This field is a member of `oneof`_ ``_excess_campaign_budget``.
            budget_recommendation (google.ads.googleads.v12.resources.types.Recommendation.CampaignBudgetRecommendation):
                Output only. The recommendation for the
                constrained budget to increase.
        """

        excess_campaign_budget = proto.Field(
            proto.STRING, number=3, optional=True,
        )
        budget_recommendation = proto.Field(
            proto.MESSAGE,
            number=2,
            message="Recommendation.CampaignBudgetRecommendation",
        )

    class TargetRoasOptInRecommendation(proto.Message):
        r"""The Target ROAS opt-in recommendation.

        Attributes:
            recommended_target_roas (float):
                Output only. The recommended target ROAS
                (revenue per unit of spend). The value is
                between 0.01 and 1000.0, inclusive.

                This field is a member of `oneof`_ ``_recommended_target_roas``.
            required_campaign_budget_amount_micros (int):
                Output only. The minimum campaign budget, in
                local currency for the account, required to
                achieve the target ROAS. Amount is specified in
                micros, where one million is equivalent to one
                currency unit.

                This field is a member of `oneof`_ ``_required_campaign_budget_amount_micros``.
        """

        recommended_target_roas = proto.Field(
            proto.DOUBLE, number=1, optional=True,
        )
        required_campaign_budget_amount_micros = proto.Field(
            proto.INT64, number=2, optional=True,
        )

    class ResponsiveSearchAdAssetRecommendation(proto.Message):
        r"""The add responsive search ad asset recommendation.

        Attributes:
            current_ad (google.ads.googleads.v12.resources.types.Ad):
                Output only. The current ad to be updated.
            recommended_assets (google.ads.googleads.v12.resources.types.Ad):
                Output only. The recommended assets. This is
                populated only with the new headlines and/or
                descriptions, and is otherwise empty.
        """

        current_ad = proto.Field(proto.MESSAGE, number=1, message=gagr_ad.Ad,)
        recommended_assets = proto.Field(
            proto.MESSAGE, number=2, message=gagr_ad.Ad,
        )

    class ResponsiveSearchAdImproveAdStrengthRecommendation(proto.Message):
        r"""The responsive search ad improve ad strength recommendation.

        Attributes:
            current_ad (google.ads.googleads.v12.resources.types.Ad):
                Output only. The current ad to be updated.
            recommended_ad (google.ads.googleads.v12.resources.types.Ad):
                Output only. The updated ad.
        """

        current_ad = proto.Field(proto.MESSAGE, number=1, message=gagr_ad.Ad,)
        recommended_ad = proto.Field(
            proto.MESSAGE, number=2, message=gagr_ad.Ad,
        )

    class ResponsiveSearchAdRecommendation(proto.Message):
        r"""The add responsive search ad recommendation.

        Attributes:
            ad (google.ads.googleads.v12.resources.types.Ad):
                Output only. Recommended ad.
        """

        ad = proto.Field(proto.MESSAGE, number=1, message=gagr_ad.Ad,)

    class UseBroadMatchKeywordRecommendation(proto.Message):
        r"""The use broad match keyword recommendation.

        Attributes:
            keyword (Sequence[google.ads.googleads.v12.common.types.KeywordInfo]):
                Output only. Sample of keywords to be
                expanded to Broad Match.
            suggested_keywords_count (int):
                Output only. Total number of keywords to be
                expanded to Broad Match in the campaign.
            campaign_keywords_count (int):
                Output only. Total number of keywords in the
                campaign.
            campaign_uses_shared_budget (bool):
                Output only. Whether the associated campaign
                uses a shared budget.
            required_campaign_budget_amount_micros (int):
                Output only. The budget recommended to avoid
                becoming budget constrained after applying the
                recommendation.
        """

        keyword = proto.RepeatedField(
            proto.MESSAGE, number=1, message=criteria.KeywordInfo,
        )
        suggested_keywords_count = proto.Field(proto.INT64, number=2,)
        campaign_keywords_count = proto.Field(proto.INT64, number=3,)
        campaign_uses_shared_budget = proto.Field(proto.BOOL, number=4,)
        required_campaign_budget_amount_micros = proto.Field(
            proto.INT64, number=5,
        )

    class UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation(
        proto.Message
    ):
        r"""The upgrade a Smart Shopping campaign to a Performance Max
        campaign recommendation.

        Attributes:
            merchant_id (int):
                Output only. ID of Merchant Center account.
            sales_country_code (str):
                Output only. Country whose products from
                merchant's inventory should be included.
        """

        merchant_id = proto.Field(proto.INT64, number=1,)
        sales_country_code = proto.Field(proto.STRING, number=2,)

    class RaiseTargetCpaBidTooLowRecommendation(proto.Message):
        r"""The raise target CPA bid too low recommendation.

        Attributes:
            recommended_target_multiplier (float):
                Output only. A number greater than 1.0
                indicating the factor by which we recommend the
                target CPA should be increased.

                This field is a member of `oneof`_ ``_recommended_target_multiplier``.
            average_target_cpa_micros (int):
                Output only. The current average target CPA
                of the campaign, in micros of customer local
                currency.

                This field is a member of `oneof`_ ``_average_target_cpa_micros``.
        """

        recommended_target_multiplier = proto.Field(
            proto.DOUBLE, number=1, optional=True,
        )
        average_target_cpa_micros = proto.Field(
            proto.INT64, number=2, optional=True,
        )

    class DisplayExpansionOptInRecommendation(proto.Message):
        r"""The Display Expansion opt-in recommendation.
        """

    class UpgradeLocalCampaignToPerformanceMaxRecommendation(proto.Message):
        r"""The Upgrade Local campaign to Performance Max campaign
        recommendation.

        """

    class ForecastingSetTargetRoasRecommendation(proto.Message):
        r"""The forecasting set target ROAS recommendation.

        Attributes:
            recommended_target_roas (float):
                Output only. The recommended target ROAS
                (revenue per unit of spend). The value is
                between 0.01 and 1000.0, inclusive.
            campaign_budget (google.ads.googleads.v12.resources.types.Recommendation.CampaignBudget):
                Output only. The campaign budget.
        """

        recommended_target_roas = proto.Field(proto.DOUBLE, number=1,)
        campaign_budget = proto.Field(
            proto.MESSAGE, number=2, message="Recommendation.CampaignBudget",
        )

    class CampaignBudget(proto.Message):
        r"""A campaign budget shared amongst various budget
        recommendation types.

        Attributes:
            current_amount_micros (int):
                Output only. Current budget amount.
            recommended_new_amount_micros (int):
                Output only. Recommended budget amount.
            new_start_date (str):
                Output only. The date when the new budget would start being
                used. This field will be set for the following
                recommendation types: FORECASTING_SET_TARGET_ROAS.
                YYYY-MM-DD format, for example, 2018-04-17.
        """

        current_amount_micros = proto.Field(proto.INT64, number=1,)
        recommended_new_amount_micros = proto.Field(proto.INT64, number=2,)
        new_start_date = proto.Field(proto.STRING, number=3,)

    resource_name = proto.Field(proto.STRING, number=1,)
    type_ = proto.Field(
        proto.ENUM,
        number=2,
        enum=recommendation_type.RecommendationTypeEnum.RecommendationType,
    )
    impact = proto.Field(proto.MESSAGE, number=3, message=RecommendationImpact,)
    campaign_budget = proto.Field(proto.STRING, number=24, optional=True,)
    campaign = proto.Field(proto.STRING, number=25, optional=True,)
    ad_group = proto.Field(proto.STRING, number=26, optional=True,)
    dismissed = proto.Field(proto.BOOL, number=27, optional=True,)
    campaign_budget_recommendation = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="recommendation",
        message=CampaignBudgetRecommendation,
    )
    forecasting_campaign_budget_recommendation = proto.Field(
        proto.MESSAGE,
        number=22,
        oneof="recommendation",
        message=CampaignBudgetRecommendation,
    )
    keyword_recommendation = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="recommendation",
        message=KeywordRecommendation,
    )
    text_ad_recommendation = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="recommendation",
        message=TextAdRecommendation,
    )
    target_cpa_opt_in_recommendation = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="recommendation",
        message=TargetCpaOptInRecommendation,
    )
    maximize_conversions_opt_in_recommendation = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="recommendation",
        message=MaximizeConversionsOptInRecommendation,
    )
    enhanced_cpc_opt_in_recommendation = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof="recommendation",
        message=EnhancedCpcOptInRecommendation,
    )
    search_partners_opt_in_recommendation = proto.Field(
        proto.MESSAGE,
        number=14,
        oneof="recommendation",
        message=SearchPartnersOptInRecommendation,
    )
    maximize_clicks_opt_in_recommendation = proto.Field(
        proto.MESSAGE,
        number=15,
        oneof="recommendation",
        message=MaximizeClicksOptInRecommendation,
    )
    optimize_ad_rotation_recommendation = proto.Field(
        proto.MESSAGE,
        number=16,
        oneof="recommendation",
        message=OptimizeAdRotationRecommendation,
    )
    callout_extension_recommendation = proto.Field(
        proto.MESSAGE,
        number=17,
        oneof="recommendation",
        message=CalloutExtensionRecommendation,
    )
    sitelink_extension_recommendation = proto.Field(
        proto.MESSAGE,
        number=18,
        oneof="recommendation",
        message=SitelinkExtensionRecommendation,
    )
    call_extension_recommendation = proto.Field(
        proto.MESSAGE,
        number=19,
        oneof="recommendation",
        message=CallExtensionRecommendation,
    )
    keyword_match_type_recommendation = proto.Field(
        proto.MESSAGE,
        number=20,
        oneof="recommendation",
        message=KeywordMatchTypeRecommendation,
    )
    move_unused_budget_recommendation = proto.Field(
        proto.MESSAGE,
        number=21,
        oneof="recommendation",
        message=MoveUnusedBudgetRecommendation,
    )
    target_roas_opt_in_recommendation = proto.Field(
        proto.MESSAGE,
        number=23,
        oneof="recommendation",
        message=TargetRoasOptInRecommendation,
    )
    responsive_search_ad_recommendation = proto.Field(
        proto.MESSAGE,
        number=28,
        oneof="recommendation",
        message=ResponsiveSearchAdRecommendation,
    )
    marginal_roi_campaign_budget_recommendation = proto.Field(
        proto.MESSAGE,
        number=29,
        oneof="recommendation",
        message=CampaignBudgetRecommendation,
    )
    use_broad_match_keyword_recommendation = proto.Field(
        proto.MESSAGE,
        number=30,
        oneof="recommendation",
        message=UseBroadMatchKeywordRecommendation,
    )
    responsive_search_ad_asset_recommendation = proto.Field(
        proto.MESSAGE,
        number=31,
        oneof="recommendation",
        message=ResponsiveSearchAdAssetRecommendation,
    )
    upgrade_smart_shopping_campaign_to_performance_max_recommendation = proto.Field(
        proto.MESSAGE,
        number=32,
        oneof="recommendation",
        message=UpgradeSmartShoppingCampaignToPerformanceMaxRecommendation,
    )
    responsive_search_ad_improve_ad_strength_recommendation = proto.Field(
        proto.MESSAGE,
        number=33,
        oneof="recommendation",
        message=ResponsiveSearchAdImproveAdStrengthRecommendation,
    )
    display_expansion_opt_in_recommendation = proto.Field(
        proto.MESSAGE,
        number=34,
        oneof="recommendation",
        message=DisplayExpansionOptInRecommendation,
    )
    upgrade_local_campaign_to_performance_max_recommendation = proto.Field(
        proto.MESSAGE,
        number=35,
        oneof="recommendation",
        message=UpgradeLocalCampaignToPerformanceMaxRecommendation,
    )
    raise_target_cpa_bid_too_low_recommendation = proto.Field(
        proto.MESSAGE,
        number=36,
        oneof="recommendation",
        message=RaiseTargetCpaBidTooLowRecommendation,
    )
    forecasting_set_target_roas_recommendation = proto.Field(
        proto.MESSAGE,
        number=37,
        oneof="recommendation",
        message=ForecastingSetTargetRoasRecommendation,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.errors",
    marshal="google.ads.googleads.v12",
    manifest={"ManagerLinkErrorEnum",},
)


class ManagerLinkErrorEnum(proto.Message):
    r"""Container for enum describing possible ManagerLink errors.
    """

    class ManagerLinkError(proto.Enum):
        r"""Enum describing possible ManagerLink errors."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        ACCOUNTS_NOT_COMPATIBLE_FOR_LINKING = 2
        TOO_MANY_MANAGERS = 3
        TOO_MANY_INVITES = 4
        ALREADY_INVITED_BY_THIS_MANAGER = 5
        ALREADY_MANAGED_BY_THIS_MANAGER = 6
        ALREADY_MANAGED_IN_HIERARCHY = 7
        DUPLICATE_CHILD_FOUND = 8
        CLIENT_HAS_NO_ADMIN_USER = 9
        MAX_DEPTH_EXCEEDED = 10
        CYCLE_NOT_ALLOWED = 11
        TOO_MANY_ACCOUNTS = 12
        TOO_MANY_ACCOUNTS_AT_MANAGER = 13
        NON_OWNER_USER_CANNOT_MODIFY_LINK = 14
        SUSPENDED_ACCOUNT_CANNOT_ADD_CLIENTS = 15
        CLIENT_OUTSIDE_TREE = 16
        INVALID_STATUS_CHANGE = 17
        INVALID_CHANGE = 18
        CUSTOMER_CANNOT_MANAGE_SELF = 19
        CREATING_ENABLED_LINK_NOT_ALLOWED = 20


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.common.types import custom_parameter
from airflow.providers.google_vendor.googleads.v12.common.types import feed_common
from airflow.providers.google_vendor.googleads.v12.common.types import policy
from airflow.providers.google_vendor.googleads.v12.enums.types import (
    feed_item_quality_approval_status,
)
from airflow.providers.google_vendor.googleads.v12.enums.types import (
    feed_item_quality_disapproval_reason,
)
from airflow.providers.google_vendor.googleads.v12.enums.types import feed_item_status
from airflow.providers.google_vendor.googleads.v12.enums.types import feed_item_validation_status
from airflow.providers.google_vendor.googleads.v12.enums.types import (
    geo_targeting_restriction as gage_geo_targeting_restriction,
)
from airflow.providers.google_vendor.googleads.v12.enums.types import placeholder_type
from airflow.providers.google_vendor.googleads.v12.enums.types import policy_approval_status
from airflow.providers.google_vendor.googleads.v12.enums.types import policy_review_status
from airflow.providers.google_vendor.googleads.v12.errors.types import feed_item_validation_error


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.resources",
    marshal="google.ads.googleads.v12",
    manifest={
        "FeedItem",
        "FeedItemAttributeValue",
        "FeedItemPlaceholderPolicyInfo",
        "FeedItemValidationError",
    },
)


class FeedItem(proto.Message):
    r"""A feed item.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the feed item. Feed item
            resource names have the form:

            ``customers/{customer_id}/feedItems/{feed_id}~{feed_item_id}``
        feed (str):
            Immutable. The feed to which this feed item
            belongs.

            This field is a member of `oneof`_ ``_feed``.
        id (int):
            Output only. The ID of this feed item.

            This field is a member of `oneof`_ ``_id``.
        start_date_time (str):
            Start time in which this feed item is
            effective and can begin serving. The time is in
            the customer's time zone. The format is
            "YYYY-MM-DD HH:MM:SS".
            Examples: "2018-03-05 09:15:00" or "2018-02-01
            14:34:30".

            This field is a member of `oneof`_ ``_start_date_time``.
        end_date_time (str):
            End time in which this feed item is no longer
            effective and will stop serving. The time is in
            the customer's time zone. The format is
            "YYYY-MM-DD HH:MM:SS".
            Examples: "2018-03-05 09:15:00" or "2018-02-01
            14:34:30".

            This field is a member of `oneof`_ ``_end_date_time``.
        attribute_values (Sequence[google.ads.googleads.v12.resources.types.FeedItemAttributeValue]):
            The feed item's attribute values.
        geo_targeting_restriction (google.ads.googleads.v12.enums.types.GeoTargetingRestrictionEnum.GeoTargetingRestriction):
            Geo targeting restriction specifies the type
            of location that can be used for targeting.
        url_custom_parameters (Sequence[google.ads.googleads.v12.common.types.CustomParameter]):
            The list of mappings used to substitute custom parameter
            tags in a ``tracking_url_template``, ``final_urls``, or
            ``mobile_final_urls``.
        status (google.ads.googleads.v12.enums.types.FeedItemStatusEnum.FeedItemStatus):
            Output only. Status of the feed item.
            This field is read-only.
        policy_infos (Sequence[google.ads.googleads.v12.resources.types.FeedItemPlaceholderPolicyInfo]):
            Output only. List of info about a feed item's
            validation and approval state for active feed
            mappings. There will be an entry in the list for
            each type of feed mapping associated with the
            feed, for example, a feed with a sitelink and a
            call feed mapping would cause every feed item
            associated with that feed to have an entry in
            this list for both sitelink and call. This field
            is read-only.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    feed = proto.Field(proto.STRING, number=11, optional=True,)
    id = proto.Field(proto.INT64, number=12, optional=True,)
    start_date_time = proto.Field(proto.STRING, number=13, optional=True,)
    end_date_time = proto.Field(proto.STRING, number=14, optional=True,)
    attribute_values = proto.RepeatedField(
        proto.MESSAGE, number=6, message="FeedItemAttributeValue",
    )
    geo_targeting_restriction = proto.Field(
        proto.ENUM,
        number=7,
        enum=gage_geo_targeting_restriction.GeoTargetingRestrictionEnum.GeoTargetingRestriction,
    )
    url_custom_parameters = proto.RepeatedField(
        proto.MESSAGE, number=8, message=custom_parameter.CustomParameter,
    )
    status = proto.Field(
        proto.ENUM,
        number=9,
        enum=feed_item_status.FeedItemStatusEnum.FeedItemStatus,
    )
    policy_infos = proto.RepeatedField(
        proto.MESSAGE, number=10, message="FeedItemPlaceholderPolicyInfo",
    )


class FeedItemAttributeValue(proto.Message):
    r"""A feed item attribute value.

    Attributes:
        feed_attribute_id (int):
            Id of the feed attribute for which the value
            is associated with.

            This field is a member of `oneof`_ ``_feed_attribute_id``.
        integer_value (int):
            Int64 value. Should be set if feed_attribute_id refers to a
            feed attribute of type INT64.

            This field is a member of `oneof`_ ``_integer_value``.
        boolean_value (bool):
            Bool value. Should be set if feed_attribute_id refers to a
            feed attribute of type BOOLEAN.

            This field is a member of `oneof`_ ``_boolean_value``.
        string_value (str):
            String value. Should be set if feed_attribute_id refers to a
            feed attribute of type STRING, URL or DATE_TIME. For STRING
            the maximum length is 1500 characters. For URL the maximum
            length is 2076 characters. For DATE_TIME the string must be
            in the format "YYYYMMDD HHMMSS".

            This field is a member of `oneof`_ ``_string_value``.
        double_value (float):
            Double value. Should be set if feed_attribute_id refers to a
            feed attribute of type DOUBLE.

            This field is a member of `oneof`_ ``_double_value``.
        price_value (google.ads.googleads.v12.common.types.Money):
            Price value. Should be set if feed_attribute_id refers to a
            feed attribute of type PRICE.
        integer_values (Sequence[int]):
            Repeated int64 value. Should be set if feed_attribute_id
            refers to a feed attribute of type INT64_LIST.
        boolean_values (Sequence[bool]):
            Repeated bool value. Should be set if feed_attribute_id
            refers to a feed attribute of type BOOLEAN_LIST.
        string_values (Sequence[str]):
            Repeated string value. Should be set if feed_attribute_id
            refers to a feed attribute of type STRING_LIST, URL_LIST or
            DATE_TIME_LIST. For STRING_LIST and URL_LIST the total size
            of the list in bytes may not exceed 3000. For DATE_TIME_LIST
            the number of elements may not exceed 200.

            For STRING_LIST the maximum length of each string element is
            1500 characters. For URL_LIST the maximum length is 2076
            characters. For DATE_TIME the format of the string must be
            the same as start and end time for the feed item.
        double_values (Sequence[float]):
            Repeated double value. Should be set if feed_attribute_id
            refers to a feed attribute of type DOUBLE_LIST.
    """

    feed_attribute_id = proto.Field(proto.INT64, number=11, optional=True,)
    integer_value = proto.Field(proto.INT64, number=12, optional=True,)
    boolean_value = proto.Field(proto.BOOL, number=13, optional=True,)
    string_value = proto.Field(proto.STRING, number=14, optional=True,)
    double_value = proto.Field(proto.DOUBLE, number=15, optional=True,)
    price_value = proto.Field(
        proto.MESSAGE, number=6, message=feed_common.Money,
    )
    integer_values = proto.RepeatedField(proto.INT64, number=16,)
    boolean_values = proto.RepeatedField(proto.BOOL, number=17,)
    string_values = proto.RepeatedField(proto.STRING, number=18,)
    double_values = proto.RepeatedField(proto.DOUBLE, number=19,)


class FeedItemPlaceholderPolicyInfo(proto.Message):
    r"""Policy, validation, and quality approval info for a feed item
    for the specified placeholder type.

    Attributes:
        placeholder_type_enum (google.ads.googleads.v12.enums.types.PlaceholderTypeEnum.PlaceholderType):
            Output only. The placeholder type.
        feed_mapping_resource_name (str):
            Output only. The FeedMapping that contains
            the placeholder type.

            This field is a member of `oneof`_ ``_feed_mapping_resource_name``.
        review_status (google.ads.googleads.v12.enums.types.PolicyReviewStatusEnum.PolicyReviewStatus):
            Output only. Where the placeholder type is in
            the review process.
        approval_status (google.ads.googleads.v12.enums.types.PolicyApprovalStatusEnum.PolicyApprovalStatus):
            Output only. The overall approval status of
            the placeholder type, calculated based on the
            status of its individual policy topic entries.
        policy_topic_entries (Sequence[google.ads.googleads.v12.common.types.PolicyTopicEntry]):
            Output only. The list of policy findings for
            the placeholder type.
        validation_status (google.ads.googleads.v12.enums.types.FeedItemValidationStatusEnum.FeedItemValidationStatus):
            Output only. The validation status of the
            palceholder type.
        validation_errors (Sequence[google.ads.googleads.v12.resources.types.FeedItemValidationError]):
            Output only. List of placeholder type
            validation errors.
        quality_approval_status (google.ads.googleads.v12.enums.types.FeedItemQualityApprovalStatusEnum.FeedItemQualityApprovalStatus):
            Output only. Placeholder type quality
            evaluation approval status.
        quality_disapproval_reasons (Sequence[google.ads.googleads.v12.enums.types.FeedItemQualityDisapprovalReasonEnum.FeedItemQualityDisapprovalReason]):
            Output only. List of placeholder type quality
            evaluation disapproval reasons.
    """

    placeholder_type_enum = proto.Field(
        proto.ENUM,
        number=10,
        enum=placeholder_type.PlaceholderTypeEnum.PlaceholderType,
    )
    feed_mapping_resource_name = proto.Field(
        proto.STRING, number=11, optional=True,
    )
    review_status = proto.Field(
        proto.ENUM,
        number=3,
        enum=policy_review_status.PolicyReviewStatusEnum.PolicyReviewStatus,
    )
    approval_status = proto.Field(
        proto.ENUM,
        number=4,
        enum=policy_approval_status.PolicyApprovalStatusEnum.PolicyApprovalStatus,
    )
    policy_topic_entries = proto.RepeatedField(
        proto.MESSAGE, number=5, message=policy.PolicyTopicEntry,
    )
    validation_status = proto.Field(
        proto.ENUM,
        number=6,
        enum=feed_item_validation_status.FeedItemValidationStatusEnum.FeedItemValidationStatus,
    )
    validation_errors = proto.RepeatedField(
        proto.MESSAGE, number=7, message="FeedItemValidationError",
    )
    quality_approval_status = proto.Field(
        proto.ENUM,
        number=8,
        enum=feed_item_quality_approval_status.FeedItemQualityApprovalStatusEnum.FeedItemQualityApprovalStatus,
    )
    quality_disapproval_reasons = proto.RepeatedField(
        proto.ENUM,
        number=9,
        enum=feed_item_quality_disapproval_reason.FeedItemQualityDisapprovalReasonEnum.FeedItemQualityDisapprovalReason,
    )


class FeedItemValidationError(proto.Message):
    r"""Stores a validation error and the set of offending feed
    attributes which together are responsible for causing a feed
    item validation error.

    Attributes:
        validation_error (google.ads.googleads.v12.errors.types.FeedItemValidationErrorEnum.FeedItemValidationError):
            Output only. Error code indicating what
            validation error was triggered. The description
            of the error can be found in the 'description'
            field.
        description (str):
            Output only. The description of the
            validation error.

            This field is a member of `oneof`_ ``_description``.
        feed_attribute_ids (Sequence[int]):
            Output only. Set of feed attributes in the
            feed item flagged during validation. If empty,
            no specific feed attributes can be associated
            with the error (for example, error across the
            entire feed item).
        extra_info (str):
            Output only. Any extra information related to this error
            which is not captured by validation_error and
            feed_attribute_id (for example, placeholder field IDs when
            feed_attribute_id is not mapped). Note that extra_info is
            not localized.

            This field is a member of `oneof`_ ``_extra_info``.
    """

    validation_error = proto.Field(
        proto.ENUM,
        number=1,
        enum=feed_item_validation_error.FeedItemValidationErrorEnum.FeedItemValidationError,
    )
    description = proto.Field(proto.STRING, number=6, optional=True,)
    feed_attribute_ids = proto.RepeatedField(proto.INT64, number=7,)
    extra_info = proto.Field(proto.STRING, number=8, optional=True,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.resources.types import (
    keyword_plan_campaign_keyword,
)
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.services",
    marshal="google.ads.googleads.v12",
    manifest={
        "MutateKeywordPlanCampaignKeywordsRequest",
        "KeywordPlanCampaignKeywordOperation",
        "MutateKeywordPlanCampaignKeywordsResponse",
        "MutateKeywordPlanCampaignKeywordResult",
    },
)


class MutateKeywordPlanCampaignKeywordsRequest(proto.Message):
    r"""Request message for
    [KeywordPlanCampaignKeywordService.MutateKeywordPlanCampaignKeywords][google.ads.googleads.v12.services.KeywordPlanCampaignKeywordService.MutateKeywordPlanCampaignKeywords].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose
            campaign keywords are being modified.
        operations (Sequence[google.ads.googleads.v12.services.types.KeywordPlanCampaignKeywordOperation]):
            Required. The list of operations to perform
            on individual Keyword Plan campaign keywords.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id = proto.Field(proto.STRING, number=1,)
    operations = proto.RepeatedField(
        proto.MESSAGE, number=2, message="KeywordPlanCampaignKeywordOperation",
    )
    partial_failure = proto.Field(proto.BOOL, number=3,)
    validate_only = proto.Field(proto.BOOL, number=4,)


class KeywordPlanCampaignKeywordOperation(proto.Message):
    r"""A single operation (create, update, remove) on a Keyword Plan
    campaign keyword.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            The FieldMask that determines which resource
            fields are modified in an update.
        create (google.ads.googleads.v12.resources.types.KeywordPlanCampaignKeyword):
            Create operation: No resource name is
            expected for the new Keyword Plan campaign
            keyword.

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v12.resources.types.KeywordPlanCampaignKeyword):
            Update operation: The Keyword Plan campaign
            keyword expected to have a valid resource name.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: A resource name for the removed Keyword
            Plan campaign keywords expected in this format:

            ``customers/{customer_id}/keywordPlanCampaignKeywords/{kp_campaign_keyword_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask = proto.Field(
        proto.MESSAGE, number=4, message=field_mask_pb2.FieldMask,
    )
    create = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=keyword_plan_campaign_keyword.KeywordPlanCampaignKeyword,
    )
    update = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=keyword_plan_campaign_keyword.KeywordPlanCampaignKeyword,
    )
    remove = proto.Field(proto.STRING, number=3, oneof="operation",)


class MutateKeywordPlanCampaignKeywordsResponse(proto.Message):
    r"""Response message for a Keyword Plan campaign keyword mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
        results (Sequence[google.ads.googleads.v12.services.types.MutateKeywordPlanCampaignKeywordResult]):
            All results for the mutate.
    """

    partial_failure_error = proto.Field(
        proto.MESSAGE, number=3, message=status_pb2.Status,
    )
    results = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="MutateKeywordPlanCampaignKeywordResult",
    )


class MutateKeywordPlanCampaignKeywordResult(proto.Message):
    r"""The result for the Keyword Plan campaign keyword mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name = proto.Field(proto.STRING, number=1,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.errors",
    marshal="google.ads.googleads.v12",
    manifest={"CampaignExperimentErrorEnum",},
)


class CampaignExperimentErrorEnum(proto.Message):
    r"""Container for enum describing possible campaign experiment
    errors.

    """

    class CampaignExperimentError(proto.Enum):
        r"""Enum describing possible campaign experiment errors."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        DUPLICATE_NAME = 2
        INVALID_TRANSITION = 3
        CANNOT_CREATE_EXPERIMENT_WITH_SHARED_BUDGET = 4
        CANNOT_CREATE_EXPERIMENT_FOR_REMOVED_BASE_CAMPAIGN = 5
        CANNOT_CREATE_EXPERIMENT_FOR_NON_PROPOSED_DRAFT = 6
        CUSTOMER_CANNOT_CREATE_EXPERIMENT = 7
        CAMPAIGN_CANNOT_CREATE_EXPERIMENT = 8
        EXPERIMENT_DURATIONS_MUST_NOT_OVERLAP = 9
        EXPERIMENT_DURATION_MUST_BE_WITHIN_CAMPAIGN_DURATION = 10
        CANNOT_MUTATE_EXPERIMENT_DUE_TO_STATUS = 11


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.enums.types import keyword_match_type


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.resources",
    marshal="google.ads.googleads.v12",
    manifest={"KeywordPlanAdGroupKeyword",},
)


class KeywordPlanAdGroupKeyword(proto.Message):
    r"""A Keyword Plan ad group keyword.
    Max number of keyword plan keywords per plan: 10000.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the Keyword Plan ad group
            keyword. KeywordPlanAdGroupKeyword resource names have the
            form:

            ``customers/{customer_id}/keywordPlanAdGroupKeywords/{kp_ad_group_keyword_id}``
        keyword_plan_ad_group (str):
            The Keyword Plan ad group to which this
            keyword belongs.

            This field is a member of `oneof`_ ``_keyword_plan_ad_group``.
        id (int):
            Output only. The ID of the Keyword Plan
            keyword.

            This field is a member of `oneof`_ ``_id``.
        text (str):
            The keyword text.

            This field is a member of `oneof`_ ``_text``.
        match_type (google.ads.googleads.v12.enums.types.KeywordMatchTypeEnum.KeywordMatchType):
            The keyword match type.
        cpc_bid_micros (int):
            A keyword level max cpc bid in micros (for
            example, $1 = 1mm). The currency is the same as
            the account currency code. This will override
            any CPC bid set at the keyword plan ad group
            level. Not applicable for negative keywords.
            (negative = true) This field is Optional.

            This field is a member of `oneof`_ ``_cpc_bid_micros``.
        negative (bool):
            Immutable. If true, the keyword is negative.

            This field is a member of `oneof`_ ``_negative``.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    keyword_plan_ad_group = proto.Field(proto.STRING, number=8, optional=True,)
    id = proto.Field(proto.INT64, number=9, optional=True,)
    text = proto.Field(proto.STRING, number=10, optional=True,)
    match_type = proto.Field(
        proto.ENUM,
        number=5,
        enum=keyword_match_type.KeywordMatchTypeEnum.KeywordMatchType,
    )
    cpc_bid_micros = proto.Field(proto.INT64, number=11, optional=True,)
    negative = proto.Field(proto.BOOL, number=12, optional=True,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.resources",
    marshal="google.ads.googleads.v12",
    manifest={"AdGroupCriterionLabel",},
)


class AdGroupCriterionLabel(proto.Message):
    r"""A relationship between an ad group criterion and a label.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the ad group criterion
            label. Ad group criterion label resource names have the
            form:
            ``customers/{customer_id}/adGroupCriterionLabels/{ad_group_id}~{criterion_id}~{label_id}``
        ad_group_criterion (str):
            Immutable. The ad group criterion to which
            the label is attached.

            This field is a member of `oneof`_ ``_ad_group_criterion``.
        label (str):
            Immutable. The label assigned to the ad group
            criterion.

            This field is a member of `oneof`_ ``_label``.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    ad_group_criterion = proto.Field(proto.STRING, number=4, optional=True,)
    label = proto.Field(proto.STRING, number=5, optional=True,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.errors",
    marshal="google.ads.googleads.v12",
    manifest={"BiddingErrorEnum",},
)


class BiddingErrorEnum(proto.Message):
    r"""Container for enum describing possible bidding errors.
    """

    class BiddingError(proto.Enum):
        r"""Enum describing possible bidding errors."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        BIDDING_STRATEGY_TRANSITION_NOT_ALLOWED = 2
        CANNOT_ATTACH_BIDDING_STRATEGY_TO_CAMPAIGN = 7
        INVALID_ANONYMOUS_BIDDING_STRATEGY_TYPE = 10
        INVALID_BIDDING_STRATEGY_TYPE = 14
        INVALID_BID = 17
        BIDDING_STRATEGY_NOT_AVAILABLE_FOR_ACCOUNT_TYPE = 18
        CONVERSION_TRACKING_NOT_ENABLED = 19
        NOT_ENOUGH_CONVERSIONS = 20
        CANNOT_CREATE_CAMPAIGN_WITH_BIDDING_STRATEGY = 21
        CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CAMPAIGN_LEVEL_POP_BIDDING_STRATEGY = (
            23
        )
        BIDDING_STRATEGY_NOT_SUPPORTED_WITH_AD_SCHEDULE = 24
        PAY_PER_CONVERSION_NOT_AVAILABLE_FOR_CUSTOMER = 25
        PAY_PER_CONVERSION_NOT_ALLOWED_WITH_TARGET_CPA = 26
        BIDDING_STRATEGY_NOT_ALLOWED_FOR_SEARCH_ONLY_CAMPAIGNS = 27
        BIDDING_STRATEGY_NOT_SUPPORTED_IN_DRAFTS_OR_EXPERIMENTS = 28
        BIDDING_STRATEGY_TYPE_DOES_NOT_SUPPORT_PRODUCT_TYPE_ADGROUP_CRITERION = (
            29
        )
        BID_TOO_SMALL = 30
        BID_TOO_BIG = 31
        BID_TOO_MANY_FRACTIONAL_DIGITS = 32
        INVALID_DOMAIN_NAME = 33
        NOT_COMPATIBLE_WITH_PAYMENT_MODE = 34
        NOT_COMPATIBLE_WITH_BUDGET_TYPE = 35
        NOT_COMPATIBLE_WITH_BIDDING_STRATEGY_TYPE = 36
        BIDDING_STRATEGY_TYPE_INCOMPATIBLE_WITH_SHARED_BUDGET = 37
        BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ALIGNED = 38
        BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ATTACHED_TO_THE_SAME_CAMPAIGNS_TO_ALIGN = (
            39
        )
        BIDDING_STRATEGY_AND_BUDGET_MUST_BE_REMOVED_TOGETHER = 40


__all__ = tuple(sorted(__protobuf__.manifest))



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import asyncio
from typing import Any, AsyncIterator

from aiohttp import ClientSession

from airflow.providers.google.cloud.hooks.gcs import GCSAsyncHook
from airflow.triggers.base import BaseTrigger, TriggerEvent


class GCSBlobTrigger(BaseTrigger):
    """
    A trigger that fires and it finds the requested file or folder present in the given bucket.

    :param bucket: the bucket in the google cloud storage where the objects are residing.
    :param object_name: the file or folder present in the bucket
    :param google_cloud_conn_id: reference to the Google Connection
    :param poke_interval: polling period in seconds to check for file/folder
    """

    def __init__(
        self,
        bucket: str,
        object_name: str,
        poke_interval: float,
        google_cloud_conn_id: str,
        hook_params: dict[str, Any],
    ):
        super().__init__()
        self.bucket = bucket
        self.object_name = object_name
        self.poke_interval = poke_interval
        self.google_cloud_conn_id: str = google_cloud_conn_id
        self.hook_params = hook_params

    def serialize(self) -> tuple[str, dict[str, Any]]:
        """Serializes GCSBlobTrigger arguments and classpath."""
        return (
            "airflow.providers.google.cloud.triggers.gcs.GCSBlobTrigger",
            {
                "bucket": self.bucket,
                "object_name": self.object_name,
                "poke_interval": self.poke_interval,
                "google_cloud_conn_id": self.google_cloud_conn_id,
                "hook_params": self.hook_params,
            },
        )

    async def run(self) -> AsyncIterator["TriggerEvent"]:
        """Simple loop until the relevant file/folder is found."""
        try:
            hook = self._get_async_hook()
            while True:
                res = await self._object_exists(
                    hook=hook, bucket_name=self.bucket, object_name=self.object_name
                )
                if res == "success":
                    yield TriggerEvent({"status": "success", "message": res})
                await asyncio.sleep(self.poke_interval)
        except Exception as e:
            yield TriggerEvent({"status": "error", "message": str(e)})
            return

    def _get_async_hook(self) -> GCSAsyncHook:
        return GCSAsyncHook(gcp_conn_id=self.google_cloud_conn_id, **self.hook_params)

    async def _object_exists(self, hook: GCSAsyncHook, bucket_name: str, object_name: str) -> str:
        """
        Checks for the existence of a file in Google Cloud Storage.

        :param bucket_name: The Google Cloud Storage bucket where the object is.
        :param object_name: The name of the blob_name to check in the Google cloud
            storage bucket.
        """
        async with ClientSession() as s:
            client = await hook.get_storage_client(s)
            bucket = client.get_bucket(bucket_name)
            object_response = await bucket.blob_exists(blob_name=object_name)
            if object_response:
                return "success"
            return "pending"



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.enums",
    marshal="google.ads.googleads.v12",
    manifest={"ConversionOrAdjustmentLagBucketEnum",},
)


class ConversionOrAdjustmentLagBucketEnum(proto.Message):
    r"""Container for enum representing the number of days between
    the impression and the conversion or between the impression and
    adjustments to the conversion.

    """

    class ConversionOrAdjustmentLagBucket(proto.Enum):
        r"""Enum representing the number of days between the impression
        and the conversion or between the impression and adjustments to
        the conversion.
        """
        UNSPECIFIED = 0
        UNKNOWN = 1
        CONVERSION_LESS_THAN_ONE_DAY = 2
        CONVERSION_ONE_TO_TWO_DAYS = 3
        CONVERSION_TWO_TO_THREE_DAYS = 4
        CONVERSION_THREE_TO_FOUR_DAYS = 5
        CONVERSION_FOUR_TO_FIVE_DAYS = 6
        CONVERSION_FIVE_TO_SIX_DAYS = 7
        CONVERSION_SIX_TO_SEVEN_DAYS = 8
        CONVERSION_SEVEN_TO_EIGHT_DAYS = 9
        CONVERSION_EIGHT_TO_NINE_DAYS = 10
        CONVERSION_NINE_TO_TEN_DAYS = 11
        CONVERSION_TEN_TO_ELEVEN_DAYS = 12
        CONVERSION_ELEVEN_TO_TWELVE_DAYS = 13
        CONVERSION_TWELVE_TO_THIRTEEN_DAYS = 14
        CONVERSION_THIRTEEN_TO_FOURTEEN_DAYS = 15
        CONVERSION_FOURTEEN_TO_TWENTY_ONE_DAYS = 16
        CONVERSION_TWENTY_ONE_TO_THIRTY_DAYS = 17
        CONVERSION_THIRTY_TO_FORTY_FIVE_DAYS = 18
        CONVERSION_FORTY_FIVE_TO_SIXTY_DAYS = 19
        CONVERSION_SIXTY_TO_NINETY_DAYS = 20
        ADJUSTMENT_LESS_THAN_ONE_DAY = 21
        ADJUSTMENT_ONE_TO_TWO_DAYS = 22
        ADJUSTMENT_TWO_TO_THREE_DAYS = 23
        ADJUSTMENT_THREE_TO_FOUR_DAYS = 24
        ADJUSTMENT_FOUR_TO_FIVE_DAYS = 25
        ADJUSTMENT_FIVE_TO_SIX_DAYS = 26
        ADJUSTMENT_SIX_TO_SEVEN_DAYS = 27
        ADJUSTMENT_SEVEN_TO_EIGHT_DAYS = 28
        ADJUSTMENT_EIGHT_TO_NINE_DAYS = 29
        ADJUSTMENT_NINE_TO_TEN_DAYS = 30
        ADJUSTMENT_TEN_TO_ELEVEN_DAYS = 31
        ADJUSTMENT_ELEVEN_TO_TWELVE_DAYS = 32
        ADJUSTMENT_TWELVE_TO_THIRTEEN_DAYS = 33
        ADJUSTMENT_THIRTEEN_TO_FOURTEEN_DAYS = 34
        ADJUSTMENT_FOURTEEN_TO_TWENTY_ONE_DAYS = 35
        ADJUSTMENT_TWENTY_ONE_TO_THIRTY_DAYS = 36
        ADJUSTMENT_THIRTY_TO_FORTY_FIVE_DAYS = 37
        ADJUSTMENT_FORTY_FIVE_TO_SIXTY_DAYS = 38
        ADJUSTMENT_SIXTY_TO_NINETY_DAYS = 39
        ADJUSTMENT_NINETY_TO_ONE_HUNDRED_AND_FORTY_FIVE_DAYS = 40
        CONVERSION_UNKNOWN = 41
        ADJUSTMENT_UNKNOWN = 42


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.errors",
    marshal="google.ads.googleads.v12",
    manifest={"SettingErrorEnum",},
)


class SettingErrorEnum(proto.Message):
    r"""Container for enum describing possible setting errors.
    """

    class SettingError(proto.Enum):
        r"""Enum describing possible setting errors."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        SETTING_TYPE_IS_NOT_AVAILABLE = 3
        SETTING_TYPE_IS_NOT_COMPATIBLE_WITH_CAMPAIGN = 4
        TARGETING_SETTING_CONTAINS_INVALID_CRITERION_TYPE_GROUP = 5
        TARGETING_SETTING_DEMOGRAPHIC_CRITERION_TYPE_GROUPS_MUST_BE_SET_TO_TARGET_ALL = (
            6
        )
        TARGETING_SETTING_CANNOT_CHANGE_TARGET_ALL_TO_FALSE_FOR_DEMOGRAPHIC_CRITERION_TYPE_GROUP = (
            7
        )
        DYNAMIC_SEARCH_ADS_SETTING_AT_LEAST_ONE_FEED_ID_MUST_BE_PRESENT = 8
        DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_DOMAIN_NAME = 9
        DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_SUBDOMAIN_NAME = 10
        DYNAMIC_SEARCH_ADS_SETTING_CONTAINS_INVALID_LANGUAGE_CODE = 11
        TARGET_ALL_IS_NOT_ALLOWED_FOR_PLACEMENT_IN_SEARCH_CAMPAIGN = 12
        SETTING_VALUE_NOT_COMPATIBLE_WITH_CAMPAIGN = 20
        BID_ONLY_IS_NOT_ALLOWED_TO_BE_MODIFIED_WITH_CUSTOMER_MATCH_TARGETING = (
            21
        )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Wrapper classes used to modify the behavior of response objects."""

import grpc
import time

from airflow.providers.google_vendor.googleads import util
from types import SimpleNamespace


class _UnaryStreamWrapper(grpc.Call, grpc.Future):
    def __init__(self, underlay_call, failure_handler, use_proto_plus=False):
        super().__init__()
        self._underlay_call = underlay_call
        self._failure_handler = failure_handler
        self._exception = None
        self._use_proto_plus = use_proto_plus
        self._cache = SimpleNamespace(**{"initial_response_object": None})

    def initial_metadata(self):
        return self._underlay_call.initial_metadata()

    def trailing_metadata(self):
        return self._underlay_call.initial_metadata()

    def code(self):
        return self._underlay_call.code()

    def details(self):
        return self._underlay_call.details()

    def debug_error_string(self):
        return self._underlay_call.debug_error_string()

    def cancelled(self):
        return self._underlay_call.cancelled()

    def running(self):
        return self._underlay_call.running()

    def done(self):
        return self._underlay_call.done()

    def result(self, timeout=None):
        return self._underlay_call.result(timeout=timeout)

    def exception(self, timeout=None):
        if self._exception:
            return self._exception
        else:
            return self._underlay_call.exception(timeout=timeout)

    def traceback(self, timeout=None):
        return self._underlay_call.traceback(timeout=timeout)

    def add_done_callback(self, fn):
        return self._underlay_call.add_done_callback(fn)

    def add_callback(self, callback):
        return self._underlay_call.add_callback(callback)

    def is_active(self):
        return self._underlay_call.is_active()

    def time_remaining(self):
        return self._underlay_call.time_remaining()

    def cancel(self):
        return self._underlay_call.cancel()

    def __iter__(self):
        return self

    def __next__(self):
        try:
            message = next(self._underlay_call)
            # Store only the first streaming response object in _cache.initial_response_object.
            # Each streaming response object captures 10,000 rows.
            # The default log character limit is 5,000, so caching multiple
            # streaming response objects does not make sense in most cases,
            # as only [part of] 1 will get logged.
            if self._cache.initial_response_object == None:
                self._cache.initial_response_object = message
            if self._use_proto_plus == True:
                # By default this message is wrapped by proto-plus
                return message
            else:
                return util.convert_proto_plus_to_protobuf(message)
        except StopIteration:
            raise
        except Exception:
            try:
                self._failure_handler(self._underlay_call)
            except Exception as e:
                self._exception = e
                raise e

    def get_cache(self):
        return self._cache


class _UnaryUnaryWrapper(grpc.Call, grpc.Future):
    def __init__(self, underlay_call, use_proto_plus=False):
        super().__init__()
        self._underlay_call = underlay_call
        self._use_proto_plus = use_proto_plus

    def initial_metadata(self):
        return self._underlay_call.initial_metadata()

    def trailing_metadata(self):
        return self._underlay_call.initial_metadata()

    def code(self):
        return self._underlay_call.code()

    def details(self):
        return self._underlay_call.details()

    def debug_error_string(self):
        return self._underlay_call.debug_error_string()

    def cancelled(self):
        return self._underlay_call.cancelled()

    def running(self):
        return self._underlay_call.running()

    def done(self):
        return self._underlay_call.done()

    def result(self, timeout=None):
        message = self._underlay_call.result()
        if self._use_proto_plus == True:
            return message
        else:
            return util.convert_proto_plus_to_protobuf(message)

    def exception(self, timeout=None):
        if self._exception:
            return self._exception
        else:
            return self._underlay_call.exception(timeout=timeout)

    def traceback(self, timeout=None):
        return self._underlay_call.traceback(timeout=timeout)

    def add_done_callback(self, fn):
        return self._underlay_call.add_done_callback(fn)

    def add_callback(self, callback):
        return self._underlay_call.add_callback(callback)

    def is_active(self):
        return self._underlay_call.is_active()

    def time_remaining(self):
        return self._underlay_call.time_remaining()

    def cancel(self):
        return self._underlay_call.cancel()

    def __iter__(self):
        if self._use_proto_plus == True:
            return self
        else:
            return util.convert_proto_plus_to_protobuf(self)

    def __next__(self):
        return next(self._underlay_call)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.enums",
    marshal="google.ads.googleads.v12",
    manifest={"RecommendationTypeEnum",},
)


class RecommendationTypeEnum(proto.Message):
    r"""Container for enum describing types of recommendations.
    """

    class RecommendationType(proto.Enum):
        r"""Types of recommendations."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        CAMPAIGN_BUDGET = 2
        KEYWORD = 3
        TEXT_AD = 4
        TARGET_CPA_OPT_IN = 5
        MAXIMIZE_CONVERSIONS_OPT_IN = 6
        ENHANCED_CPC_OPT_IN = 7
        SEARCH_PARTNERS_OPT_IN = 8
        MAXIMIZE_CLICKS_OPT_IN = 9
        OPTIMIZE_AD_ROTATION = 10
        CALLOUT_EXTENSION = 11
        SITELINK_EXTENSION = 12
        CALL_EXTENSION = 13
        KEYWORD_MATCH_TYPE = 14
        MOVE_UNUSED_BUDGET = 15
        FORECASTING_CAMPAIGN_BUDGET = 16
        TARGET_ROAS_OPT_IN = 17
        RESPONSIVE_SEARCH_AD = 18
        MARGINAL_ROI_CAMPAIGN_BUDGET = 19
        USE_BROAD_MATCH_KEYWORD = 20
        RESPONSIVE_SEARCH_AD_ASSET = 21
        UPGRADE_SMART_SHOPPING_CAMPAIGN_TO_PERFORMANCE_MAX = 22
        RESPONSIVE_SEARCH_AD_IMPROVE_AD_STRENGTH = 23
        DISPLAY_EXPANSION_OPT_IN = 24
        UPGRADE_LOCAL_CAMPAIGN_TO_PERFORMANCE_MAX = 25
        RAISE_TARGET_CPA_BID_TOO_LOW = 26
        FORECASTING_SET_TARGET_ROAS = 27


__all__ = tuple(sorted(__protobuf__.manifest))



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import warnings
from tempfile import NamedTemporaryFile
from typing import TYPE_CHECKING, Sequence

from airflow.exceptions import AirflowException
from airflow.providers.amazon.aws.hooks.s3 import S3Hook
from airflow.providers.google.cloud.hooks.gcs import GCSHook, _parse_gcs_url, gcs_object_is_directory

try:
    from airflow.providers.amazon.aws.operators.s3 import S3ListOperator
except ImportError:
    from airflow.providers.amazon.aws.operators.s3_list import S3ListOperator  # type: ignore[no-redef]

if TYPE_CHECKING:
    from airflow.utils.context import Context


class S3ToGCSOperator(S3ListOperator):
    """
    Synchronizes an S3 key, possibly a prefix, with a Google Cloud Storage
    destination path.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:S3ToGCSOperator`

    :param bucket: The S3 bucket where to find the objects. (templated)
    :param prefix: Prefix string which filters objects whose name begin with
        such prefix. (templated)
    :param delimiter: the delimiter marks key hierarchy. (templated)
    :param aws_conn_id: The source S3 connection
    :param verify: Whether or not to verify SSL certificates for S3 connection.
        By default SSL certificates are verified.
        You can provide the following values:

        - ``False``: do not validate SSL certificates. SSL will still be used
                 (unless use_ssl is False), but SSL certificates will not be
                 verified.
        - ``path/to/cert/bundle.pem``: A filename of the CA cert bundle to uses.
                 You can specify this argument if you want to use a different
                 CA cert bundle than the one used by botocore.
    :param gcp_conn_id: (Optional) The connection ID used to connect to Google Cloud.
    :param dest_gcs: The destination Google Cloud Storage bucket and prefix
        where you want to store the files. (templated)
    :param delegate_to: Google account to impersonate using domain-wide delegation of authority,
        if any. For this to work, the service account making the request must have
        domain-wide delegation enabled.
    :param replace: Whether you want to replace existing destination files
        or not.
    :param gzip: Option to compress file for upload
    :param google_impersonation_chain: Optional Google service account to impersonate using
        short-term credentials, or chained list of accounts required to get the access_token
        of the last account in the list, which will be impersonated in the request.
        If set as a string, the account must grant the originating account
        the Service Account Token Creator IAM role.
        If set as a sequence, the identities from the list must grant
        Service Account Token Creator IAM role to the directly preceding identity, with first
        account from the list granting this role to the originating account (templated).


    **Example**:

    .. code-block:: python

       s3_to_gcs_op = S3ToGCSOperator(
           task_id="s3_to_gcs_example",
           bucket="my-s3-bucket",
           prefix="data/customers-201804",
           gcp_conn_id="google_cloud_default",
           dest_gcs="gs://my.gcs.bucket/some/customers/",
           replace=False,
           gzip=True,
           dag=my_dag,
       )

    Note that ``bucket``, ``prefix``, ``delimiter`` and ``dest_gcs`` are
    templated, so you can use variables in them if you wish.
    """

    template_fields: Sequence[str] = (
        "bucket",
        "prefix",
        "delimiter",
        "dest_gcs",
        "google_impersonation_chain",
    )
    ui_color = "#e09411"

    def __init__(
        self,
        *,
        bucket,
        prefix="",
        delimiter="",
        aws_conn_id="aws_default",
        verify=None,
        gcp_conn_id="google_cloud_default",
        dest_gcs=None,
        delegate_to=None,
        replace=False,
        gzip=False,
        google_impersonation_chain: str | Sequence[str] | None = None,
        **kwargs,
    ):

        super().__init__(bucket=bucket, prefix=prefix, delimiter=delimiter, aws_conn_id=aws_conn_id, **kwargs)
        self.gcp_conn_id = gcp_conn_id
        self.dest_gcs = dest_gcs
        if delegate_to:
            warnings.warn(
                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
            )
        self.delegate_to = delegate_to
        self.replace = replace
        self.verify = verify
        self.gzip = gzip
        self.google_impersonation_chain = google_impersonation_chain

    def _check_inputs(self) -> None:
        if self.dest_gcs and not gcs_object_is_directory(self.dest_gcs):
            self.log.info(
                "Destination Google Cloud Storage path is not a valid "
                '"directory", define a path that ends with a slash "/" or '
                "leave it empty for the root of the bucket."
            )
            raise AirflowException(
                'The destination Google Cloud Storage path must end with a slash "/" or be empty.'
            )

    def execute(self, context: Context):
        self._check_inputs()
        # use the super method to list all the files in an S3 bucket/key
        files = super().execute(context)

        gcs_hook = GCSHook(
            gcp_conn_id=self.gcp_conn_id,
            delegate_to=self.delegate_to,
            impersonation_chain=self.google_impersonation_chain,
        )

        if not self.replace:
            # if we are not replacing -> list all files in the GCS bucket
            # and only keep those files which are present in
            # S3 and not in Google Cloud Storage
            bucket_name, object_prefix = _parse_gcs_url(self.dest_gcs)
            existing_files_prefixed = gcs_hook.list(bucket_name, prefix=object_prefix)

            existing_files = []

            if existing_files_prefixed:
                # Remove the object prefix itself, an empty directory was found
                if object_prefix in existing_files_prefixed:
                    existing_files_prefixed.remove(object_prefix)

                # Remove the object prefix from all object string paths
                for f in existing_files_prefixed:
                    if f.startswith(object_prefix):
                        existing_files.append(f[len(object_prefix) :])
                    else:
                        existing_files.append(f)

            files = list(set(files) - set(existing_files))
            if len(files) > 0:
                self.log.info("%s files are going to be synced: %s.", len(files), files)
            else:
                self.log.info("There are no new files to sync. Have a nice day!")

        if files:
            hook = S3Hook(aws_conn_id=self.aws_conn_id, verify=self.verify)

            for file in files:
                # GCS hook builds its own in-memory file so we have to create
                # and pass the path
                file_object = hook.get_key(file, self.bucket)
                with NamedTemporaryFile(mode="wb", delete=True) as f:
                    file_object.download_fileobj(f)
                    f.flush()

                    dest_gcs_bucket, dest_gcs_object_prefix = _parse_gcs_url(self.dest_gcs)
                    # There will always be a '/' before file because it is
                    # enforced at instantiation time
                    dest_gcs_object = dest_gcs_object_prefix + file

                    # Sync is sequential and the hook already logs too much
                    # so skip this for now
                    # self.log.info(
                    #     'Saving file {0} from S3 bucket {1} in GCS bucket {2}'
                    #     ' as object {3}'.format(file, self.bucket,
                    #                             dest_gcs_bucket,
                    #                             dest_gcs_object))

                    gcs_hook.upload(dest_gcs_bucket, dest_gcs_object, f.name, gzip=self.gzip)

            self.log.info("All done, uploaded %d files to Google Cloud Storage", len(files))
        else:
            self.log.info("In sync, no files needed to be uploaded to Google Cloud Storage")

        return files



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.enums.types import (
    feed_item_set_string_filter_type,
)


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.common",
    marshal="google.ads.googleads.v12",
    manifest={
        "DynamicLocationSetFilter",
        "BusinessNameFilter",
        "DynamicAffiliateLocationSetFilter",
    },
)


class DynamicLocationSetFilter(proto.Message):
    r"""Represents a filter on locations in a feed item set.
    Only applicable if the parent Feed of the FeedItemSet is a
    LOCATION feed.

    Attributes:
        labels (Sequence[str]):
            If multiple labels are set, then only
            feeditems marked with all the labels will be
            added to the FeedItemSet.
        business_name_filter (google.ads.googleads.v12.common.types.BusinessNameFilter):
            Business name filter.
    """

    labels = proto.RepeatedField(proto.STRING, number=1,)
    business_name_filter = proto.Field(
        proto.MESSAGE, number=2, message="BusinessNameFilter",
    )


class BusinessNameFilter(proto.Message):
    r"""Represents a business name filter on locations in a
    FeedItemSet.

    Attributes:
        business_name (str):
            Business name string to use for filtering.
        filter_type (google.ads.googleads.v12.enums.types.FeedItemSetStringFilterTypeEnum.FeedItemSetStringFilterType):
            The type of string matching to use when filtering with
            business_name.
    """

    business_name = proto.Field(proto.STRING, number=1,)
    filter_type = proto.Field(
        proto.ENUM,
        number=2,
        enum=feed_item_set_string_filter_type.FeedItemSetStringFilterTypeEnum.FeedItemSetStringFilterType,
    )


class DynamicAffiliateLocationSetFilter(proto.Message):
    r"""Represents a filter on affiliate locations in a FeedItemSet. Only
    applicable if the parent Feed of the FeedItemSet is an
    AFFILIATE_LOCATION feed.

    Attributes:
        chain_ids (Sequence[int]):
            Used to filter affiliate locations by chain
            ids. Only affiliate locations that belong to the
            specified chain(s) will be added to the
            FeedItemSet.
    """

    chain_ids = proto.RepeatedField(proto.INT64, number=1,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.common.types import extensions
from airflow.providers.google_vendor.googleads.v12.enums.types import keyword_match_type
from airflow.providers.google_vendor.googleads.v12.resources.types import ad as gagr_ad
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.services",
    marshal="google.ads.googleads.v12",
    manifest={
        "ApplyRecommendationRequest",
        "ApplyRecommendationOperation",
        "ApplyRecommendationResponse",
        "ApplyRecommendationResult",
        "DismissRecommendationRequest",
        "DismissRecommendationResponse",
    },
)


class ApplyRecommendationRequest(proto.Message):
    r"""Request message for
    [RecommendationService.ApplyRecommendation][google.ads.googleads.v12.services.RecommendationService.ApplyRecommendation].

    Attributes:
        customer_id (str):
            Required. The ID of the customer with the
            recommendation.
        operations (Sequence[google.ads.googleads.v12.services.types.ApplyRecommendationOperation]):
            Required. The list of operations to apply recommendations.
            If partial_failure=false all recommendations should be of
            the same type There is a limit of 100 operations per
            request.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, operations will be carried out
            as a transaction if and only if they are all
            valid. Default is false.
    """

    customer_id = proto.Field(proto.STRING, number=1,)
    operations = proto.RepeatedField(
        proto.MESSAGE, number=2, message="ApplyRecommendationOperation",
    )
    partial_failure = proto.Field(proto.BOOL, number=3,)


class ApplyRecommendationOperation(proto.Message):
    r"""Information about the operation to apply a recommendation and
    any parameters to customize it.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            The resource name of the recommendation to
            apply.
        campaign_budget (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.CampaignBudgetParameters):
            Optional parameters to use when applying a
            campaign budget recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        text_ad (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.TextAdParameters):
            Optional parameters to use when applying a
            text ad recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        keyword (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.KeywordParameters):
            Optional parameters to use when applying
            keyword recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        target_cpa_opt_in (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.TargetCpaOptInParameters):
            Optional parameters to use when applying
            target CPA opt-in recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        target_roas_opt_in (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.TargetRoasOptInParameters):
            Optional parameters to use when applying
            target ROAS opt-in recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        callout_extension (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.CalloutExtensionParameters):
            Parameters to use when applying callout
            extension recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        call_extension (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.CallExtensionParameters):
            Parameters to use when applying call
            extension recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        sitelink_extension (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.SitelinkExtensionParameters):
            Parameters to use when applying sitelink
            extension recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        move_unused_budget (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.MoveUnusedBudgetParameters):
            Parameters to use when applying move unused
            budget recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        responsive_search_ad (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.ResponsiveSearchAdParameters):
            Parameters to use when applying a responsive
            search ad recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        use_broad_match_keyword (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.UseBroadMatchKeywordParameters):
            Parameters to use when applying a use broad
            match keyword recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        responsive_search_ad_asset (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters):
            Parameters to use when applying a responsive
            search ad asset recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        responsive_search_ad_improve_ad_strength (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters):
            Parameters to use when applying a responsive
            search ad improve ad strength recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
        raise_target_cpa_bid_too_low (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.RaiseTargetCpaBidTooLowParameters):
            Parameters to use when applying a raise
            target CPA bid too low recommendation. The apply
            is asynchronous and can take minutes depending
            on the number of ad groups there is in the
            related campaign.

            This field is a member of `oneof`_ ``apply_parameters``.
        forecasting_set_target_roas (google.ads.googleads.v12.services.types.ApplyRecommendationOperation.ForecastingSetTargetRoasParameters):
            Parameters to use when applying a forecasting
            set target ROAS recommendation.

            This field is a member of `oneof`_ ``apply_parameters``.
    """

    class CampaignBudgetParameters(proto.Message):
        r"""Parameters to use when applying a campaign budget
        recommendation.

        Attributes:
            new_budget_amount_micros (int):
                New budget amount to set for target budget
                resource. This is a required field.

                This field is a member of `oneof`_ ``_new_budget_amount_micros``.
        """

        new_budget_amount_micros = proto.Field(
            proto.INT64, number=2, optional=True,
        )

    class ForecastingSetTargetRoasParameters(proto.Message):
        r"""Parameters to use when applying a forecasting set target roas
        recommendation.

        Attributes:
            target_roas (float):
                New target ROAS (revenue per unit of spend)
                to set for a campaign resource.
                The value is between 0.01 and 1000.0, inclusive.

                This field is a member of `oneof`_ ``_target_roas``.
            campaign_budget_amount_micros (int):
                New campaign budget amount to set for a
                campaign resource.

                This field is a member of `oneof`_ ``_campaign_budget_amount_micros``.
        """

        target_roas = proto.Field(proto.DOUBLE, number=1, optional=True,)
        campaign_budget_amount_micros = proto.Field(
            proto.INT64, number=2, optional=True,
        )

    class TextAdParameters(proto.Message):
        r"""Parameters to use when applying a text ad recommendation.

        Attributes:
            ad (google.ads.googleads.v12.resources.types.Ad):
                New ad to add to recommended ad group. All
                necessary fields need to be set in this message.
                This is a required field.
        """

        ad = proto.Field(proto.MESSAGE, number=1, message=gagr_ad.Ad,)

    class KeywordParameters(proto.Message):
        r"""Parameters to use when applying keyword recommendation.

        Attributes:
            ad_group (str):
                The ad group resource to add keyword to. This
                is a required field.

                This field is a member of `oneof`_ ``_ad_group``.
            match_type (google.ads.googleads.v12.enums.types.KeywordMatchTypeEnum.KeywordMatchType):
                The match type of the keyword. This is a
                required field.
            cpc_bid_micros (int):
                Optional, CPC bid to set for the keyword. If
                not set, keyword will use bid based on bidding
                strategy used by target ad group.

                This field is a member of `oneof`_ ``_cpc_bid_micros``.
        """

        ad_group = proto.Field(proto.STRING, number=4, optional=True,)
        match_type = proto.Field(
            proto.ENUM,
            number=2,
            enum=keyword_match_type.KeywordMatchTypeEnum.KeywordMatchType,
        )
        cpc_bid_micros = proto.Field(proto.INT64, number=5, optional=True,)

    class TargetCpaOptInParameters(proto.Message):
        r"""Parameters to use when applying Target CPA recommendation.

        Attributes:
            target_cpa_micros (int):
                Average CPA to use for Target CPA bidding
                strategy. This is a required field.

                This field is a member of `oneof`_ ``_target_cpa_micros``.
            new_campaign_budget_amount_micros (int):
                Optional, budget amount to set for the
                campaign.

                This field is a member of `oneof`_ ``_new_campaign_budget_amount_micros``.
        """

        target_cpa_micros = proto.Field(proto.INT64, number=3, optional=True,)
        new_campaign_budget_amount_micros = proto.Field(
            proto.INT64, number=4, optional=True,
        )

    class TargetRoasOptInParameters(proto.Message):
        r"""Parameters to use when applying a Target ROAS opt-in
        recommendation.

        Attributes:
            target_roas (float):
                Average ROAS (revenue per unit of spend) to use for Target
                ROAS bidding strategy. The value is between 0.01 and 1000.0,
                inclusive. This is a required field, unless
                new_campaign_budget_amount_micros is set.

                This field is a member of `oneof`_ ``_target_roas``.
            new_campaign_budget_amount_micros (int):
                Optional, budget amount to set for the
                campaign.

                This field is a member of `oneof`_ ``_new_campaign_budget_amount_micros``.
        """

        target_roas = proto.Field(proto.DOUBLE, number=1, optional=True,)
        new_campaign_budget_amount_micros = proto.Field(
            proto.INT64, number=2, optional=True,
        )

    class CalloutExtensionParameters(proto.Message):
        r"""Parameters to use when applying callout extension
        recommendation.

        Attributes:
            callout_extensions (Sequence[google.ads.googleads.v12.common.types.CalloutFeedItem]):
                Callout extensions to be added. This is a
                required field.
        """

        callout_extensions = proto.RepeatedField(
            proto.MESSAGE, number=1, message=extensions.CalloutFeedItem,
        )

    class CallExtensionParameters(proto.Message):
        r"""Parameters to use when applying call extension
        recommendation.

        Attributes:
            call_extensions (Sequence[google.ads.googleads.v12.common.types.CallFeedItem]):
                Call extensions to be added. This is a
                required field.
        """

        call_extensions = proto.RepeatedField(
            proto.MESSAGE, number=1, message=extensions.CallFeedItem,
        )

    class SitelinkExtensionParameters(proto.Message):
        r"""Parameters to use when applying sitelink extension
        recommendation.

        Attributes:
            sitelink_extensions (Sequence[google.ads.googleads.v12.common.types.SitelinkFeedItem]):
                Sitelink extensions to be added. This is a
                required field.
        """

        sitelink_extensions = proto.RepeatedField(
            proto.MESSAGE, number=1, message=extensions.SitelinkFeedItem,
        )

    class MoveUnusedBudgetParameters(proto.Message):
        r"""Parameters to use when applying move unused budget
        recommendation.

        Attributes:
            budget_micros_to_move (int):
                Budget amount to move from excess budget to
                constrained budget. This is a required field.

                This field is a member of `oneof`_ ``_budget_micros_to_move``.
        """

        budget_micros_to_move = proto.Field(
            proto.INT64, number=2, optional=True,
        )

    class ResponsiveSearchAdAssetParameters(proto.Message):
        r"""Parameters to use when applying a responsive search ad asset
        recommendation.

        Attributes:
            updated_ad (google.ads.googleads.v12.resources.types.Ad):
                Updated ad. The current ad's content will be
                replaced.
        """

        updated_ad = proto.Field(proto.MESSAGE, number=1, message=gagr_ad.Ad,)

    class ResponsiveSearchAdImproveAdStrengthParameters(proto.Message):
        r"""Parameters to use when applying a responsive search ad
        improve ad strength recommendation.

        Attributes:
            updated_ad (google.ads.googleads.v12.resources.types.Ad):
                Updated ad. The current ad's content will be
                replaced.
        """

        updated_ad = proto.Field(proto.MESSAGE, number=1, message=gagr_ad.Ad,)

    class ResponsiveSearchAdParameters(proto.Message):
        r"""Parameters to use when applying a responsive search ad
        recommendation.

        Attributes:
            ad (google.ads.googleads.v12.resources.types.Ad):
                Required. New ad to add to recommended ad
                group.
        """

        ad = proto.Field(proto.MESSAGE, number=1, message=gagr_ad.Ad,)

    class RaiseTargetCpaBidTooLowParameters(proto.Message):
        r"""Parameters to use when applying a raise target CPA bid too
        low recommendation. The apply is asynchronous and can take
        minutes depending on the number of ad groups there is in the
        related campaign..

        Attributes:
            target_multiplier (float):
                Required. A number greater than 1.0
                indicating the factor by which to increase the
                target CPA. This is a required field.
        """

        target_multiplier = proto.Field(proto.DOUBLE, number=1,)

    class UseBroadMatchKeywordParameters(proto.Message):
        r"""Parameters to use when applying a use broad match keyword
        recommendation.

        Attributes:
            new_budget_amount_micros (int):
                New budget amount to set for target budget
                resource.

                This field is a member of `oneof`_ ``_new_budget_amount_micros``.
        """

        new_budget_amount_micros = proto.Field(
            proto.INT64, number=1, optional=True,
        )

    resource_name = proto.Field(proto.STRING, number=1,)
    campaign_budget = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="apply_parameters",
        message=CampaignBudgetParameters,
    )
    text_ad = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="apply_parameters",
        message=TextAdParameters,
    )
    keyword = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="apply_parameters",
        message=KeywordParameters,
    )
    target_cpa_opt_in = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="apply_parameters",
        message=TargetCpaOptInParameters,
    )
    target_roas_opt_in = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="apply_parameters",
        message=TargetRoasOptInParameters,
    )
    callout_extension = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="apply_parameters",
        message=CalloutExtensionParameters,
    )
    call_extension = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="apply_parameters",
        message=CallExtensionParameters,
    )
    sitelink_extension = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="apply_parameters",
        message=SitelinkExtensionParameters,
    )
    move_unused_budget = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="apply_parameters",
        message=MoveUnusedBudgetParameters,
    )
    responsive_search_ad = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="apply_parameters",
        message=ResponsiveSearchAdParameters,
    )
    use_broad_match_keyword = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof="apply_parameters",
        message=UseBroadMatchKeywordParameters,
    )
    responsive_search_ad_asset = proto.Field(
        proto.MESSAGE,
        number=13,
        oneof="apply_parameters",
        message=ResponsiveSearchAdAssetParameters,
    )
    responsive_search_ad_improve_ad_strength = proto.Field(
        proto.MESSAGE,
        number=14,
        oneof="apply_parameters",
        message=ResponsiveSearchAdImproveAdStrengthParameters,
    )
    raise_target_cpa_bid_too_low = proto.Field(
        proto.MESSAGE,
        number=15,
        oneof="apply_parameters",
        message=RaiseTargetCpaBidTooLowParameters,
    )
    forecasting_set_target_roas = proto.Field(
        proto.MESSAGE,
        number=16,
        oneof="apply_parameters",
        message=ForecastingSetTargetRoasParameters,
    )


class ApplyRecommendationResponse(proto.Message):
    r"""Response message for
    [RecommendationService.ApplyRecommendation][google.ads.googleads.v12.services.RecommendationService.ApplyRecommendation].

    Attributes:
        results (Sequence[google.ads.googleads.v12.services.types.ApplyRecommendationResult]):
            Results of operations to apply
            recommendations.
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors) we return
            the RPC level error.
    """

    results = proto.RepeatedField(
        proto.MESSAGE, number=1, message="ApplyRecommendationResult",
    )
    partial_failure_error = proto.Field(
        proto.MESSAGE, number=2, message=status_pb2.Status,
    )


class ApplyRecommendationResult(proto.Message):
    r"""The result of applying a recommendation.

    Attributes:
        resource_name (str):
            Returned for successful applies.
    """

    resource_name = proto.Field(proto.STRING, number=1,)


class DismissRecommendationRequest(proto.Message):
    r"""Request message for
    [RecommendationService.DismissRecommendation][google.ads.googleads.v12.services.RecommendationService.DismissRecommendation].

    Attributes:
        customer_id (str):
            Required. The ID of the customer with the
            recommendation.
        operations (Sequence[google.ads.googleads.v12.services.types.DismissRecommendationRequest.DismissRecommendationOperation]):
            Required. The list of operations to dismiss recommendations.
            If partial_failure=false all recommendations should be of
            the same type There is a limit of 100 operations per
            request.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, operations will be carried in
            a single transaction if and only if they are all
            valid. Default is false.
    """

    class DismissRecommendationOperation(proto.Message):
        r"""Operation to dismiss a single recommendation identified by
        resource_name.

        Attributes:
            resource_name (str):
                The resource name of the recommendation to
                dismiss.
        """

        resource_name = proto.Field(proto.STRING, number=1,)

    customer_id = proto.Field(proto.STRING, number=1,)
    operations = proto.RepeatedField(
        proto.MESSAGE, number=3, message=DismissRecommendationOperation,
    )
    partial_failure = proto.Field(proto.BOOL, number=2,)


class DismissRecommendationResponse(proto.Message):
    r"""Response message for
    [RecommendationService.DismissRecommendation][google.ads.googleads.v12.services.RecommendationService.DismissRecommendation].

    Attributes:
        results (Sequence[google.ads.googleads.v12.services.types.DismissRecommendationResponse.DismissRecommendationResult]):
            Results of operations to dismiss
            recommendations.
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors) we return
            the RPC level error.
    """

    class DismissRecommendationResult(proto.Message):
        r"""The result of dismissing a recommendation.

        Attributes:
            resource_name (str):
                Returned for successful dismissals.
        """

        resource_name = proto.Field(proto.STRING, number=1,)

    results = proto.RepeatedField(
        proto.MESSAGE, number=1, message=DismissRecommendationResult,
    )
    partial_failure_error = proto.Field(
        proto.MESSAGE, number=2, message=status_pb2.Status,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
